<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * abook_local_file.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * Backend for addressbook as a pipe separated file
 *
 * An array with the following elements must be passed to
 * the class constructor (elements marked ? are optional):
 *
 *    filename  => path to addressbook file
 *  ? create    => if true: file is created if it does not exist.
 *  ? umask     => umask set before opening file.
 *
 * NOTE. This class should not be used directly. Use the
 *       "AddressBook" class instead.
 *
 * $Id: group_abook_local_file.php,v 1.3 2004/06/02 07:40:39 marco Exp $
 */

if (!defined('SM_PATH')) { define('SM_PATH','../'); }

class group_abook_local_file extends addressbook_backend {
    var $btype = 'local';
    var $bname = 'local_file';

    var $filename   = '';
    var $filehandle = 0;
    var $create     = false;
    var $umask;

    /* ========================== Private ======================= */

    /* Constructor */
    function group_abook_local_file($param) {
        $this->sname = _("Personal group addressbook");
        $this->umask = Umask();

        if(is_array($param)) {
            if(empty($param['filename'])) {
                return $this->set_error('Invalid parameters');
            }
            if(!is_string($param['filename'])) {
                return $this->set_error($param['filename'] . ': '.
                     _("Not a file name"));
            }

            $this->filename = $param['filename'];

            if($param['create']) {
                $this->create = true;
            }
            if(isset($param['umask'])) {
                $this->umask = $param['umask'];
            }
            if(!empty($param['name'])) {
                $this->sname = $param['name'];
            }
           
            $this->open(true);
        } else {
            $this->set_error('Invalid argument to constructor');
        }
    }

    /* Open the addressbook file and store the file pointer.
     * Use $file as the file to open, or the class' own 
     * filename property. If $param is empty and file is  
     * open, do nothing. */
    function open($new = false) {
        $this->error = '';
        $file   = $this->filename;
        $create = $this->create;
  
        /* Return true is file is open and $new is unset */
        if($this->filehandle && !$new) { return true; }
  
        /* Check that new file exitsts */
        if((!(file_exists($file) && is_readable($file))) && !$create) {
            return $this->set_error("$file: " . _("No such file or directory"));
        }
  
        /* Close old file, if any */
        if($this->filehandle) { $this->close(); }
        
        /* Open file. First try to open for reading and writing,
         * but fall back to read only. */
        umask($this->umask);
        $fh = @fopen($file, 'a+');
        if($fh) {
            $this->filehandle = &$fh;
            $this->filename   = $file;
            $this->writeable  = true;
        } else {
            $fh = @fopen($file, 'r');
            if($fh) {
                $this->filehandle = &$fh;
                $this->filename   = $file;
                $this->writeable  = false;
            } else {
                return $this->set_error("$file: " . _("Open failed"));
            }
        }
        return true;
    }

    /* Close the file and forget the filehandle */
    function close() {
        @fclose($this->filehandle);
        $this->filehandle = 0;
        $this->filename   = '';
        $this->writable   = false;
    }

    /* Lock the datafile - try 20 times in 5 seconds */
    function lock() {
        for($i = 0 ; $i < 20 ; $i++) {
            if(flock($this->filehandle, 2 + 4)) 
                return true;
            else
                usleep(250000);
        }
        return false;
    }

    /* Lock the datafile */
    function unlock() {
        return flock($this->filehandle, 3);
    }

    /* Overwrite the file with data from $rows
     * NOTE! Previous locks are broken by this function */
    function overwrite(&$rows) {
        $this->unlock();
        $newfh = @fopen($this->filename.'.tmp', 'w');

        if(!$newfh) {
            return $this->set_error($this->filename. '.tmp:' . _("Open failed"));
        }
        
        for($i = 0, $cnt=sizeof($rows) ; $i < $cnt ; $i++) {
            if(is_array($rows[$i])) {
                for($j = 0, $cnt_part=count($rows[$i]) ; $j < $cnt_part ; $j++) {
                    $rows[$i][$j] = $this->quotevalue($rows[$i][$j]);
                }
                $tmpwrite = @fwrite($newfh, join('|', $rows[$i]) . "\n");
                if ($tmpwrite == -1) {
                    return $this->set_error($this->filename . '.tmp:' . _("Write failed"));
                }
            }
        }       

        fclose($newfh);
        if (!@copy($this->filename . '.tmp' , $this->filename)) {
          return $this->set_error($this->filename . ':' . _("Unable to update"));
        }
        @unlink($this->filename . '.tmp');
	chmod($this->filename, 0600);
        $this->unlock();
        $this->open(true);
        return true;
    }
    
    /* ========================== Public ======================== */
    
    /* Lookup alias */
    function test_id($userdata) {
	global $fieldarraygroupabook;	
	if (!empty($userdata['group_name'])) { $alias = strtolower($userdata['group_name']); }
	else { $alias=""; }
        $this->open();
        @rewind($this->filehandle);
	$idmax = 0;
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
	    if ($row[$fieldarraygroupabook["group_id"]["idfile"]] > $idmax) { $idmax=$row[$fieldarraygroupabook["group_id"]["idfile"]]; }
            if (!empty($alias) && strtolower($row[$fieldarraygroupabook["group_name"]["idfile"]]) == $alias && (!$userdata['group_id'] || $userdata['group_id'] != $row[$fieldarraygroupabook["group_id"]["idfile"]])) {
                $arow = array();
                foreach ($fieldarraygroupabook as $k => $id) {
                    $arow[$k] = $row[$id['idfile']];
                }
                $arow['backend'] = $this->bnum;
                $arow['source'] = &$this->sname;
		return array($arow,'');
            }
        }
        return array('',$idmax);
    }

   /* Lookup alias */
    function lookup($alias) {
        global $fieldarraygroupabook;
        if (!$alias) { return array(''); }
        $this->open();
        @rewind($this->filehandle);
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
            if (strtolower($row[$fieldarraygroupabook["group_name"]["idfile"]]) == strtolower($alias)) {
                $arow = array();
                foreach ($fieldarraygroupabook as $k => $id) {
                    $arow[$k] = $row[$id['idfile']];
                }
                $arow['backend'] = $this->bnum;
                $arow['source'] = &$this->sname;
                return $arow;
            }
        }
        return array('');
    }

    /* Lookup alias */
    function lookup_id($alias) {
	global $fieldarraygroupabook;
        $this->open();
        @rewind($this->filehandle);
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
            if (strtolower($row[$fieldarraygroupabook["group_id"]["idfile"]]) == strtolower($alias)) {
                $arow = array();
                foreach ($fieldarraygroupabook as $k => $id) {
                    $arow[$k] = $row[$id['idfile']];
                }
                $arow['backend'] = $this->bnum;
                $arow['source'] = &$this->sname;
		return $arow;
            }
        }
        return array('');
    }

    /* List all addresses */
    function list_addr() {
	global $fieldarraygroupabook;
        $res = array();
        $this->open();
        @rewind($this->filehandle);
        while ($row = @fgetcsv($this->filehandle, 2048, '|')) {
	    $arow = array();
	    foreach ($fieldarraygroupabook as $k => $id) {
		$arow[$k] = $row[$id['idfile']];
	    }
	    $arow['backend'] = $this->bnum;
	    $arow['source'] = &$this->sname;
	    if ($arow['group_id']) { array_push($res,$arow); }
        }
        return $res;
    }

    /* Add address */
    function add($userdata) {
	global $fieldarraygroupabook;
        if(!$this->writeable) {
            return $this->set_error(_("Group Addressbook is read-only"));
        }
        /* return id nuovo */
        $return = $this->test_id($userdata);
	$ret=$return[0];
	$idmax=$return[1] + 1;    
        if(!empty($ret)) {
            return $this->set_error(sprintf(_("Group Name '%s' already exist"),
                   $ret['group_name']));
        }
	$arow = array();
	foreach ($fieldarraygroupabook as $k => $id) {
	    $arow[$id['idfile']] = $k;
	}
	$lastk = count($arow);
	ksort ($arow);
	reset ($arow);
	$lastk = count($arow);
	$data = "";
	foreach ($arow as $k => $id) {
	    if ($id == "group_id") { $data .= $idmax; } 
	    else { $data .= $this->quotevalue($userdata[$id]); }
	    if ($k != $lastk) { $data .= '|'; }
	}

        /* Strip linefeeds */
        $data = ereg_replace("[\r\n]", ' ', $data);
        /* Add linefeed at end */
        $data = $data . "\n";
  
        /* Reopen file, just to be sure */
        $this->open(true);
        if(!$this->writeable) {
            return $this->set_error(_("Group Addressbook is read-only"));
        }
  
        /* Lock the file */
        if(!$this->lock()) {
            return $this->set_error(_("Could not lock datafile"));
        }
  
        /* Write */
        $r = fwrite($this->filehandle, $data);

        /* Unlock file */
        $this->unlock();

	chmod($this->filename, 0600);  

        /* Test write result and exit if OK */
        if($r > 0) return true;
  
        /* Fail */
        $this->set_error(_("Write to addressbook failed"));
        return false;
    }

    /* Delete address */
    function remove($alias) {
	global $fieldarraygroupabook;
        if(!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }
        
        /* Lock the file to make sure we're the only process working
         * on it. */
        if(!$this->lock()) {
            return $this->set_error(_("Could not lock datafile"));
        }
  
        /* Read file into memory, ignoring nicknames to delete */
        @rewind($this->filehandle);
        $i = 0;
        $rows = array();
        while($row = @fgetcsv($this->filehandle, 2048, '|')) {
            if($row[$fieldarraygroupabook["group_id"]["idfile"]] != $alias) {
                $rows[$i++] = $row;
            }
        }
  
        /* Write data back */
        if(!$this->overwrite($rows)) {
            $this->unlock();
            return false;
        }
  
        $this->unlock();
        return true;
    }

    /* Delete address */
    function remove_contacts($alias,$public = 0) {
	global $fieldarraygroupabook;
    	if (!$this->writeable) {
    	    return $this->set_error(_("Addressbook is read-only"));
    	}
    	/* Lock the file to make sure we're the only process working
        * on it. */
    	if(!$this->lock()) {
    	    return $this->set_error(_("Could not lock datafile"));
    	}
    	/* Read file into memory, ignoring nicknames to delete */
    	@rewind($this->filehandle);
    	$i = 0;
    	$rows = array();
    	while($row = @fgetcsv($this->filehandle, 2048, '|')) {
	    // contatto privato: cerco su gruppi privato dell'utente
	    if ($public == 0) { $key = $fieldarraygroupabook["group_list_private"]["idfile"]; }
	    else if ($public == 1) { $key = $fieldarraygroupabook["group_list_public"]["idfile"]; } 
	    $ids = $row[$key];
	    $ids = ",".$ids.",";
    	    if( preg_match("/,".$alias.",/",$ids)) {
		$ids = preg_replace('/'.$alias.',/','',$ids);
		$ids = preg_replace('/^,/','',$ids);
		$ids = preg_replace('/,$/','',$ids);
		$row[$key] = $ids;
            	$rows[$i++] = $row;
    	    } else {
		$rows[$i++] = $row;
	    }
    	}
    	/* Write data back */
    	if(!$this->overwrite($rows)) {
    	    $this->unlock();
    	    return false;
    	}
    	$this->unlock();
        return true;
    }

    /* Modify address */
    function modify($alias, $userdata) {
	global $fieldarraygroupabook;
        if(!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }
  
        /* See if user exists */
        $ret = $this->lookup($alias);
        if(empty($ret)) {
            return $this->set_error(sprintf(_("Group '%s' does not exist"),
                $userdata['group_name']));
        }
	
        /* See if user exists already */
        $return = $this->test_id($userdata);
	$ret=$return[0];
	$idmax=$return[1] + 1;    
        if(!empty($ret)) {
            return $this->set_error(sprintf(_("Group Name '%s' already exist"),
                   $ret['group_name']));
        }
  
        /* Lock the file to make sure we're the only process working
         * on it. */
        if(!$this->lock()) {
            return $this->set_error(_("Could not lock datafile"));
        }
  
        /* Read file into memory, modifying the data for the 
         * user identified by $alias */
        $this->open(true);
        @rewind($this->filehandle);
        $i = 0;
        $rows = array();
        while($row = @fgetcsv($this->filehandle, 2048, '|')) {
            if(strtolower($row[$fieldarraygroupabook["group_id"]["idfile"]]) != strtolower($alias)) {
                $rows[$i++] = $row;
            } else {
		$arow = array();
		foreach ($fieldarraygroupabook as $k => $id) {
		    $arow[$id['idfile']] = $userdata[$k];
		}
		$rows[$i++] = $arow;
            }
        }
  
        /* Write data back */
        if(!$this->overwrite($rows)) {
            $this->unlock();
            return false;
        }
  
        $this->unlock();
        return true;
    }
     
    /* Function for quoting values before saving */
    function quotevalue($value) {
        /* Quote the field if it contains | or ". Double quotes need to
         * be replaced with "" */
        if(ereg("[|\"]", $value)) {
            $value = '"' . str_replace('"', '""', $value) . '"';
        }
        return $value;
    }

} /* End of class abook_local_file */
?>