<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/
/**
 * gpg_hook_functions.php
 * -----------
 * GPG plugin hook functions file, included by setup.php.
 * Updated to account for SM 1.4 pathing issues
 *
 * Copyright (c) 2002-2003 Braverock Ventures
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * @package gpg
 * @author Brian Peterson
 * @author Aaron van Meerten
 *
 *
 * $Id: gpg_hook_functions.php,v 1.62 2003/12/19 21:18:05 ke Exp $
 *
 */
if (!defined (SM_PATH)){
    if (file_exists('./gpg_functions.php')){
        define (SM_PATH , '../../');
    } elseif (file_exists('../plugins/gpg/gpg_functions.php')) {
        define (SM_PATH, '../');
    } elseif (file_exists('../gpg_functions.php')){
        define (SM_PATH , '../../../');
    } elseif (file_exists('../../plugins/gpg/gpg_functions.php')){
        define (SM_PATH , '../../../../');
    } else { echo "unable to define SM_PATH in GPG Plugin setup.php, exiting abnormally"; }
}

require_once(SM_PATH.'plugins/gpg/gpg_pref_functions.php');
require_once(SM_PATH.'plugins/gpg/gpg_functions.php');
require_once(SM_PATH.'functions/strings.php');
require_once(SM_PATH.'functions/prefs.php');
require_once(SM_PATH.'functions/forms.php');
require_once(SM_PATH.'functions/global.php');

if ( !check_php_version(4,1) ) {
    global $_SESSION;
}

/*********************************************************************/
/**
 * function download_entity
 *
 * Downloads an entity (part of a message) and returns a temporary filename for it
 *
 * @param imapConnection $imap_stream connection to imap server
 * @param mailbox $mailbox
 * @param string $id of the message
 * @param string $ent_id entity id, which part of the message to download
 * @param string $filename optional filename to download to
 * @return array $return with $return['filename'] and $return['errors']
 */

function download_entity($imap_stream, $mailbox, $id, $ent_id, $filename='',$passbody=false,$debug=0) {
        sqGetGlobalVar('messages',$msgcollection, SQ_SESSION);
        $return['errors'] = array();
        $mbx_response =  sqimap_mailbox_select($imap_stream, $mailbox);
        $body = mime_fetch_body($imap_stream, $id, $ent_id);
        $mymsg = sqimap_get_message($imap_stream,$id, $mailbox);
        $mymsg= &$mymsg->getEntity($ent_id);
        $header = $mymsg->header;
        $type0 = $header->type0;
        $type1 = $header->type1;
        $encoding = strtolower($header->encoding);
    if ($debug) { echo "Downloading data encoded $encoding<br>\n"; }
        if ($encoding == 'base64') {
        if ($debug) { echo "Decoding body<br>\n$body<br>\n"; }
               $body = base64_decode($body);
        if ($debug) { echo "Into body<br>\n$body<br>\n"; }
        }
    if ($filename) { $tempfile = $filename; }
        else { $tempfile = getTempFile(false); }
        if (is_file($tempfile)) {
                unlink($tempfile);
        }
        $fhandle = fopen($tempfile, "wb");
        if (!fwrite($fhandle, $body, strlen($body))) {
                $return['errors'][] = _("Could not write to temporary file: ") .  $tempfile;
        } else {
                fclose($fhandle);
                $return['filename'] = $tempfile;
        if ($passbody) { $return['body'] = $body; }
        }
        return $return;
}


/*********************************************************************/
/**
 * function gpg_decrypt_attachment_do
 *
 * Hook function for application/gpg-encrypted attachments
 * This function looks for a -----BEGIN PGP MESSAGE---- and will pop a link if found
 *
 * @param array $attachinfo array passed by squirrelmail to attachment handlers
 * @return void
 */

function gpg_decrypt_attachment_do(&$attachinfo) {
    global $imapConnection,$base_uri;
        $actionlinks =& $attachinfo[1];
        $startMessage = $attachinfo[2];
        $msgid        = $attachinfo[3];
        $urlMailbox   = $attachinfo[4];
    $mailbox      = urldecode($urlMailbox);
        $entid        = $attachinfo[5];
        $defaulturl  =& $attachinfo[6];
    $dlfilename = urlencode($attachinfo[7]);

    $return = download_entity($imapConnection, $mailbox, $msgid, $entid,'',true);
    if (!(strpos($return['body'],"-----BEGIN PGP MESSAGE-----") === false)) {
            $actionlinks['gpg']['href']=$base_uri."plugins/gpg/gpg_decrypt_attach.php?passed_id=$msgid&passed_ent_id=$entid&mailbox=$urlMailbox&dlfilename=$dlfilename";
            $actionlinks['gpg']['text']=_("Decrypt Attachment");
	    $attachinfo[6] = 'location.href="'.$actionlinks['gpg']['href'].'"';
    }
}

/*********************************************************************/
/**
 * function gpg_read_body_signature_do
 *
 * This function is called by setup.php, within a hook
 * to initialize our read_body_header functions.
 *
 * @return void
 */
function gpg_read_body_signature_do($ante_body) {

    //pull in the globals we need from the read_body.php page
    global $imapConnection, $passed_id, $passed_ent_id, $mailbox;
    
    sqgetGlobalVar('decrypt_info_message', $decrypt_info_message, SQ_SESSION);
    if (is_array($decrypt_info_message)) {
	if (!$decrypt_info_message['plaintext']) {
	    $my_ante = codeHTML(_("Encrypted"));
	} else if ($decrypt_info_message['signature']) {
	    $my_ante = implode($decrypt_info_message['signature'],"\n<br>");
	} else {
	    $my_ante = codeHTML(_("Unsigned"));
	}
        $ante_body .= '</td></tr><tr height="1%"><td align="left" class="nav">';
	$ante_body .=  "<b>" . codeHTML(_("Signature")) . ":&nbsp;</b>";
	$ante_body .= $my_ante;
	$ante_body .= '<BR>';
        if (is_array($decrypt_info_message['warnings'])) {
            if (count($decrypt_info_message['warnings']) > 0) {
                $ante_body .= "<b>" . codeHTML(_("Warnings:")) . "</b><ul>\n";
                foreach ($decrypt_info_message['warnings'] as $warning) {
                    $ante_body .=  '<li>'.codeHTML($warning).'</li>';
                };
                $ante_body .=  '</ul>';
            }
        }
        if (is_array($decrypt_info_message['errors'])) {
            if (count($decrypt_info_message['errors']) > 0) {
                $ante_body .=  "<b>" . codeHTML(_("Errors:")) . "</b><ul>\n";
                foreach ($decrypt_info_message['errors'] as $errors) {
                    $ante_body .=  '<li>'.codeHTML($errors).'</li>';
                };
                $ante_body .=  '</ul>';
            }
        }
        if (is_array($decrypt_info_message['info'])) {
            if (count($decrypt_info_message['info']) > 0) {
                $ante_body .=  "<b>" .codeHTML(_("Info:")) . "</b><ul>\n";
                foreach ($decrypt_info_message['info'] as $info) {
                    $ante_body .=  '<li>'.codeHTML($info).'</li>';
                };
                $ante_body .=  '</ul>';
            }
        } 
	$ante_body .= "</td></tr><tr><td>";
   } else {
	//populate the message body
	$message = sqimap_get_message ($imapConnection, $passed_id, $mailbox);

	if ($message->body_part == "") {
	    $body = gpg_fetch_full_body ($imapConnection, $passed_id, $mailbox);
	} else {
	    $body = $message->body_part;
	}
	//check to see if we need to change encoding on body
	if (strtolower($message->header->encoding)=='quoted-printable') {
	    $body = quoted_printable_decode($body);
	}

	//check to see if we need to verify a gpg signature
	if( $message->header->type0 == "multipart" && $message->header->type1 == "signed"
    	    && $message->header->getParameter("protocol") == "application/pgp-signature" ) {
    	    $ante_body .= gpg_check_sign_pgp_mime(&$message,$body);
	} else {
    	    $ante_body .= gpg_check_sign($body);
	}
    }
    return $ante_body;
}

/*********************************************************************/
/**
 * function gpg_read_body_header_do
 *
 * This function is called by setup.php, within a hook
 * to initialize our read_body_header functions.
 *
 * @return void
 */
function gpg_read_body_header_do () {

    //pull in the globals we need from the read_body.php page
    global $imapConnection, $passed_id, $passed_ent_id, $mailbox;

    //populate the message body
    $message = sqimap_get_message ($imapConnection, $passed_id, $mailbox);

    if ($message->body_part == "") {
	$body = gpg_fetch_full_body ($imapConnection, $passed_id, $mailbox);
    } else {
	$body = $message->body_part;
    }
    //check to see if we need to change encoding on body
    if (strtolower($message->header->encoding)=='quoted-printable') {
	$body = quoted_printable_decode($body);
    }

    //and then check to see if we should register the decrypt now link
    gpg_decrypt_link($body);
    //and check to see if we should show the import link
    gpg_import_link($body);

    return;
}

/*********************************************************************/
/**
 * function gpg_printer_friendly_top_do
 */
function gpg_printer_friendly_top_do () {

    //pull in the globals we need from the read_body.php page
    global $imapConnection, $passed_id, $passed_ent_id, $mailbox;

    //populate the message body
    $message = sqimap_get_message ($imapConnection, $passed_id, $mailbox);

    if ($message->body_part == "") {
	$body = gpg_fetch_full_body ($imapConnection, $passed_id, $mailbox);
    } else {
	$body = $message->body_part;
    }
    //check to see if we need to change encoding on body
    if (strtolower($message->header->encoding)=='quoted-printable') {
	$body = quoted_printable_decode($body);
    }

    //and then check to see if we should register the decrypt now link
    gpg_decrypt_link_nav($body);
    //and check to see if we should show the import link
    gpg_import_link_nav($body);

    return;
}

/*********************************************************************/
/**
 * function gpg_fetch_full_body
 *
 * This function is called before
 * gpg_check_sign and gpg_decrypt_link
 * to retrieve the messge body for those functions
 *
 * @param string  $mailbox          mailbox
 * @param integer $imapConnection   handle to the IMAP connection
 * @param integer $passed_id        message ID to retreive from the IMAP folder
 * @return string $body as string
 */
function gpg_fetch_full_body ($imapConnection, $passed_id, $mailbox) {
   //retrieve the message body
   global $uid_support;

   $body=sqimap_run_command ($imapConnection, "FETCH $passed_id BODY[]", true, $a, $b,$uid_support);
   if (is_array($body)) {
       $body = implode($body,'');
   };

   return ($body);
}

/*********************************************************************/
/**
 * function gpg_get_message_body_imap
 *
 * get the body of a given part of a message
 *
 * @param class   &$message         Message object to get the body for
 * @param integer $imapConnection   handle to the IMAP connection
 * @param integer $passed_id        message ID to retreive from the IMAP folder
 * @return void (body stuffed into message object)
 *
 */
function gpg_get_message_body_imap( &$message, $imapConnection, $passed_id )
{
    global $uid_support;

    $message->body_part = mime_fetch_body( $imapConnection, $passed_id, $message->entity_id );
    $message->decoded_body = decodeBody( $message->body_part, $message->header->encoding );
}

/*********************************************************************/
/**
 * function gpg_get_mime_header
 *
 * download just the mime header from imap
 * (note: this does not setup a MessageHeader class
 *
 * @param class   $message          Message object to get the entity out of
 * @param integer $imapConnection   handle to the IMAP connection
 * @param integer $passed_id        message ID to retreive from the IMAP folder
 * @return string $header           RFC 822 header to parse for mime type
 *
 */
function gpg_get_mime_header ( $message, $imapConnection, $passed_id )
{
    global $uid_support;
    $read = sqimap_run_command ($imapConnection, "FETCH $passed_id BODY[".$message->entity_id.".MIME]", true, $response, $errmessage, $uid_support);
    if( is_array( $read ) )
    {
        // this loop is stolen from mime_fetch_body(...)
        do
        {
            $topline = trim(array_shift($read));
        } while($topline && ($topline[0] == '*') && !preg_match('/\* [0-9]+ FETCH.*/i', $topline)) ;
        $header = implode('', $read );
    }
    return $header;
}

/*********************************************************************/
/**
 * function gpg_decrypt_link
 *
 * This function is called by the read_body hook (above)
 *
 * Use to see if the message contains an encrypted body
 * if the message contains encrypted text, display a link to the decrypt code.
 *
 * @param string $body
 * @return void
 */
function gpg_decrypt_link($body) {
    global $passed_ent_id, $passed_id, $mailbox,$mbx_response;
    global $data_dir;
    global $safe_data_dir;
    global $username, $base_uri, $custom_css;
    global $startMessage, $sort;
    global $prev, $next;
    $safe_data_dir = getHashedDir($username,$data_dir) . DIRECTORY_SEPARATOR;
    // set this to 1 if you want to double check the prefs loading
    // $debug=0;
    load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_system_defaults.txt',$debug);
    load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_local_prefs.txt',$debug);

    $gpg_key_dir  = "$safe_data_dir$username.gnupg";
    $secring = "$gpg_key_dir/secring.gpg";

    require_once(SM_PATH . 'plugins/gpg/gpg_functions.php');
    $allowprivatekeys = $GLOBALS['GPG_SYSTEM_OPTIONS']['allowprivatekeys'];

    $passphrase_cached = 'false';

    if (gpg_is_passphrase_cached()) {
            $passphrase_cached = 'true';
    }; //end passphrase caching check

    $messageblock = '';
    //double check for a an encrypted body before displaying
    $sep = '-----BEGIN PGP MESSAGE-----';
    $end = '-----END PGP MESSAGE-----';
    $pos = strpos($body,$sep);
    if ($pos !== false) {
        $epos = strpos($body,$end,$pos);
	$epos = $epos + strlen($end);
        $messageblock = substr($body,$pos,$epos-$pos);
	sqsession_register($messageblock,'gpg_encrypted_message');
    }
    if ($pos !== false) {
	 sqgetGlobalVar('decrypt_info_message', $decrypt_info_message, SQ_SESSION);
	if (is_array($decrypt_info_message)) {
	    echo  '<TD class="titlemsgattach" rowspan="3" width="1%" align="right" valign="middle">
		<IMG SRC="'.$base_uri.'images/'.$custom_css.'/decrypt_done.gif" TITLE="'.codeHTML(_("Decrypted message")).'" BORDER=0>';
            return;
	}
        //system admin turned off private keys
        if ($allowprivatekeys =='false') {
	    $msg = jsspecialchars(_("The System Administrator has disabled private key functions"));
	    echo  '<TD class="titlemsgattach" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'titlemsgattach\'"
		onClick=\'alert("'.$msg.'")\' rowspan="3" width="1%" align="right" valign="middle">
		<IMG SRC="'.$base_uri.'images/'.$custom_css.'/decrypt.gif" BORDER=0>';
            return;
        };
        //we don't have a secret keyring
        if ((!file_exists($secring)) or (filesize($secring)==0)) {
	    $msg = jsspecialchars(_("This message contains encrypted content. To decrypt this message inside Overlook setup the GPG Plugin with a secret key"));
	    echo '<TD class="titlemsgattach" rowspan="3" width="1%" align="right" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'titlemsgattach\'" valign="middle"
		onclick=\'alert("'.$msg.'")\'>
		<IMG SRC="'.$base_uri.'images/'.$custom_css.'/decrypt.gif" BORDER=0>';
            return ;
        };
        //display the Decrypt Now link
	echo addForm($base_uri . 'src/read_body.php','POST','decrypt');
	echo addHidden('passphrase',$passphrase_cached);
	echo addHidden('passed_ent_id',$passed_ent_id);
	echo addHidden('passed_id',$passed_id);
	echo addHidden('mailbox',$mailbox);
	echo addHidden('prevmsg',$prev);
	echo addHidden('nextmsg',$next);
	echo addHidden('startMessage',$startMessage);
	echo addHidden('sort',$sort);
	echo addHidden('decrypt_do',1);
        echo "<script type='text/javascript'>
    		function gpg_decrypt_check(objform){
        	    if (objform.passphrase.value=='true') {
            		objform.submit();
        	    } else {
			new_win_overlook('Decrypt_Now','" . $base_uri . "plugins/gpg/gpg_pop_init.php?MOD=passpop&psaction=decrypt',300,200,1,200,200); 
		    }
		}
	    </script>";
	echo '<TD class="titlemsgattach" rowspan="3" width="1%" align="right" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'titlemsgattach\'" valign="middle"
		onclick="gpg_decrypt_check(document.decrypt)">
		<IMG SRC="'.$base_uri.'images/'.$custom_css.'/decrypt.gif" TITLE="'._("Decrypt Message Now").'" BORDER=0>';
	echo '</form></td>';
    }; //end double check for encrypted body
};
//"
/*********************************************************************/
/**
 * function gpg_decrypt_link_nav
 */
function gpg_decrypt_link_nav($body) {
    global $passed_ent_id, $passed_id, $mailbox,$mbx_response;
    global $data_dir;
    global $safe_data_dir;
    global $username, $base_uri, $custom_css;
    global $startMessage, $sort;
    global $prev, $next, $icon_folder;
    $safe_data_dir = getHashedDir($username,$data_dir) . DIRECTORY_SEPARATOR;
    // set this to 1 if you want to double check the prefs loading
    // $debug=0;
    load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_system_defaults.txt',$debug);
    load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_local_prefs.txt',$debug);

    $gpg_key_dir  = "$safe_data_dir$username.gnupg";
    $secring = "$gpg_key_dir/secring.gpg";

    require_once(SM_PATH . 'plugins/gpg/gpg_functions.php');
    $allowprivatekeys = $GLOBALS['GPG_SYSTEM_OPTIONS']['allowprivatekeys'];

    $passphrase_cached = 'false';

    if (gpg_is_passphrase_cached()) {
            $passphrase_cached = 'true';
    }; //end passphrase caching check

    $messageblock = '';
    //double check for a an encrypted body before displaying
    $sep = '-----BEGIN PGP MESSAGE-----';
    $end = '-----END PGP MESSAGE-----';
    $pos = strpos($body,$sep);
    if ($pos !== false) {
        $epos = strpos($body,$end,$pos);
	$epos = $epos + strlen($end);
        $messageblock = substr($body,$pos,$epos-$pos);
	sqsession_register($messageblock,'gpg_encrypted_message');
    }
    if ($pos !== false) {
	 sqgetGlobalVar('decrypt_info_message', $decrypt_info_message, SQ_SESSION);
	if (is_array($decrypt_info_message)) {
    	    echo '<td nowrap=1 class="nav" height="100%" align="center" width="70">';
        	if ($icon_folder) {
            	    echo '<img src="'.$base_uri.'images/'.$custom_css.'/decrypt_done.gif" border=0 TITLE="'._("Decrypted message").'">';
            	    echo '<br>';
        	}
        	echo codeHTML(_("Decrypted"));
    	    echo '</td>'."\n";
	    sqsession_register('','decrypt_info_message');
            return;
	}
        //system admin turned off private keys
        if ($allowprivatekeys =='false') {
	    $msg = jsspecialchars(_("The System Administrator has disabled private key functions"));
    	    echo '<td nowrap=1 class="nav" height="100%" align="center" width="70" onClick=\'alert("'.$msg.'")\' onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'">';
        	if ($icon_folder) {
            	    echo '<img src="'.$base_uri.'images/'.$custom_css.'/decrypt.gif" border=0 TITLE="'._("Decrypt message").'">';
            	    echo '<br>';
        	}
        	echo codeHTML(_("Decrypt"));
    	    echo '</td>'."\n";
            return;
        };
        //we don't have a secret keyring
        if ((!file_exists($secring)) or (filesize($secring)==0)) {
	    $msg = jsspecialchars(_("This message contains encrypted content. To decrypt this message inside Overlook setup the GPG Plugin with a secret key"));
    	    echo '<td nowrap=1 class="nav" height="100%" align="center" width="70" onClick=\'alert("'.$msg.'")\' onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'">';
        	if ($icon_folder) {
            	    echo '<img src="'.$base_uri.'images/'.$custom_css.'/decrypt.gif" border=0 TITLE="'._("Decrypt message").'">';
            	    echo '<br>';
        	}
        	echo codeHTML(_("Decrypt"));
    	    echo '</td>'."\n";
            return ;
        };
        //display the Decrypt Now link
        echo '<script language="javascript" type="text/javascript" src="'.$base_uri.'js/new_window.js"></script>';
	echo addForm($base_uri . 'src/printer_friendly_bottom.php','POST','decrypt','','','','bottom_frame');
	echo addHidden('passphrase',$passphrase_cached);
	echo addHidden('passed_ent_id',$passed_ent_id);
	echo addHidden('passed_id',$passed_id);
	echo addHidden('mailbox',$mailbox);
	echo addHidden('prevmsg',$prev);
	echo addHidden('nextmsg',$next);
	echo addHidden('startMessage',$startMessage);
	echo addHidden('sort',$sort);
	echo addHidden('decrypt_do',1);
        echo "<script type='text/javascript'>
    		function gpg_decrypt_check(objform){
        	    if (objform.passphrase.value=='true') {
            		objform.submit();
        	    } else {
			new_win_overlook('Decrypt_Now','" . $base_uri . "plugins/gpg/gpg_pop_init.php?MOD=passpop&psaction=decrypt',300,200,1,200,200);
		    }
		}
	    </script>";
    	    echo '<td nowrap=1 class="nav" height="100%" align="center" width="70" onClick="gpg_decrypt_check(document.decrypt)" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'">';
        	if ($icon_folder) {
            	    echo '<img src="'.$base_uri.'images/'.$custom_css.'/decrypt.gif" border=0 TITLE="'._("Decrypt message").'">';
            	    echo '<br>';
        	}
        	echo codeHTML(_("Decrypt"));
    	    echo '</td>'."\n";
	echo '</form>';
    }; //end double check for encrypted body
};

/*********************************************************************/
/**
 * function gpg_import_link
 *
 * This function is called by the read_body hook (above)
 *
 * Use to see if the message contains an encrypted body
 * if the message contains encrypted text, display a link to the decrypt code.
 *
 * @param string $body from gpg_fetch_full_body fn
 * @return void
 */
function gpg_import_link($body) {
    global $passed_id, $mailbox;
    global $data_dir ,$base_uri, $custom_css;
    global $safe_data_dir;
    global $username;
    $safe_data_dir=getHashedDir($username,$data_dir) . DIRECTORY_SEPARATOR;
    $gpg_key_dir ="$safe_data_dir$username.gnupg";
    $sep = '-----BEGIN PGP PUBLIC KEY BLOCK-----';
    $end = '-----END PGP PUBLIC KEY BLOCK-----';
    $pos = strpos($body,$sep);
    if ($pos !== false) {
        while ($pos !== false) {
            $epos = strpos($body,$end,$pos);
            $keyblock = substr($body,$pos,($epos+34)-$pos);
            $import_body .= $keyblock . "\n";
            $pos = strpos($body,$sep,$epos);
        }
          // show some details about this key

          // display the Import Now link
	echo addForm($base_uri . "/plugins/gpg/modules/keyring_main.php",'POST','my_add_sign','','','','options');
	echo addHidden("keystring",$import_body);
	echo addHidden("textadd",_("Import Key contained in this email now"));
	echo '</form>';
	echo  '<TD class="titlemsgattach" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'titlemsgattach\'"
		onClick="parent.new_win_overlook(\'options\',\'\',700,600,1,0,0); document.my_add_sign.submit();" rowspan="3" width="1%" align="right" valign="middle">
		<IMG SRC="'.$base_uri.'images/'.$custom_css.'/importsign.gif" BORDER=0 TITLE="'._("Import Key contained in this email now").'">';
    }
}
//"
/*********************************************************************/
/**
 * function gpg_import_link_nav
 */
function gpg_import_link_nav($body) {
    global $passed_id, $mailbox;
    global $data_dir ,$base_uri, $custom_css;
    global $safe_data_dir;
    global $username,$icon_folder;
    $safe_data_dir=getHashedDir($username,$data_dir) . DIRECTORY_SEPARATOR;
    $gpg_key_dir ="$safe_data_dir$username.gnupg";
    $sep = '-----BEGIN PGP PUBLIC KEY BLOCK-----';
    $end = '-----END PGP PUBLIC KEY BLOCK-----';
    $pos = strpos($body,$sep);
    if ($pos !== false) {
        while ($pos !== false) {
            $epos = strpos($body,$end,$pos);
            $keyblock = substr($body,$pos,($epos+34)-$pos);
            $import_body .= $keyblock . "\n";
            $pos = strpos($body,$sep,$epos);
        }
          // show some details about this key
	echo addForm($base_uri . "/plugins/gpg/modules/keyring_main.php",'POST','my_add_sign','','','','options');
	echo addHidden("keystring",$import_body);
	echo addHidden("textadd",_("Import Key contained in this email now"));
	echo '</form>';
        echo '<td nowrap=1 class="nav" height="100%" align="center" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'" width="70" onclick="parent.opener.new_win_overlook(\'options\',\'\',700,600,1,0,0); document.my_add_sign.submit();">';
            if ($icon_folder) {
                echo '<img src="'.$base_uri.'images/'.$custom_css.'/importsign.gif" border=0 TITLE="'._("Import Key contained in this email now").'">';
                echo '<br>';
            }
            echo codeHTML(_("Import key"));
        echo '</td>'."\n";
    }
}

/*********************************************************************/
/**
 * function gpg_check_sign
 *
 * This function is called by the read_body hook (above)
 *
 * @param string $body
 * @return integer 0|1
 */
function gpg_check_sign($body,$encrypted=false) {
  $return=0;
  // attempt to check the signature if it looks like it has one
  global $data_dir;
  global $username;
  global $safe_data_dir;
  $safe_data_dir=getHashedDir($username,$data_dir) . DIRECTORY_SEPARATOR;
  $gpg_key_dir ="$safe_data_dir$username.gnupg";
  $sep = '-----BEGIN PGP SIGNED MESSAGE-----';
  $esep = '-----END PGP SIGNATURE-----';
  $my_ante = '';
  $pos = strpos($body,$sep);
  if ($pos !== false) {
    $return = 1;
    $epos = strpos($body,$esep,$strpos);
    if ($epos !== false) {
	$signedbody = substr($body,$pos,$epos+strlen($esep));
        $body = escapeshellarg($signedbody);
        // set this to 1 if you want to double check the prefs loading
        $debug=0;
        load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_system_defaults.txt',$debug);
        load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_local_prefs.txt',$debug);

        $path_to_gpg = $GLOBALS['GPG_SYSTEM_OPTIONS']['path_to_gpg'];

        // 'Corporate' shared system keyring setup
        $extra_cmd = '';
        $trust_system_keyring = getPref($data_dir, $username, 'trust_system_keyring');
        $systemkeyring = $GLOBALS['GPG_SYSTEM_OPTIONS']['systemkeyring'];
        if ($systemkeyring=='true' and $trust_system_keyring == 'true') {
            $system_keyring_file = $GLOBALS['GPG_SYSTEM_OPTIONS']['systemkeyringfile'];
            $systemtrustedkey    = escapeshellarg($GLOBALS['GPG_SYSTEM_OPTIONS']['systemtrustedkey']);
            //now add the parameters to $extra_cmd
            if (is_file($system_keyring_file)) {
                $system_keyring_file = escapeshellarg($system_keyring_file);
                $extra_cmd .= " --keyring $system_keyring_file ";
                if ($systemtrustedkey != '') {
                    $extra_cmd .= " --trusted-key $systemtrustedkey ";
                };
            } elseif ($debug) echo "\n".'<br>system_keyring_file '.$system_keyring_file.' failed is_file test';
        }; //end shared system keyring

        $command = "echo $body | $path_to_gpg --batch --no-tty --homedir $gpg_key_dir $extra_cmd --verify 2>&1";
        exec($command, $results, $returnval);
        $return=gpg_parse_output($results);
	if ($return['untrusted'] == 'true') {
	    gpg_update_trustdb($debug);
	    $results='';
	    exec($command, $results, $returnval);
	    $return=gpg_parse_output($results);
	}

        if (is_array($return['signature'])) {
            foreach ($return['signature'] as $line) {
        	$my_ante .= htmlspecialchars($line);
                $my_ante .= "<br>";
            }
        }
    } 
  } else {
    $encrypted = strpos($body,'-----BEGIN PGP MESSAGE-----');
    if ($encrypted) { $my_ante .= codeHTML(_("Encrypted")); } 
  }
  if ($my_ante){
    $ante_body = '</td></tr><tr height="1%"><td align="left" class="nav">';    
    $ante_body .=  "<b>" . codeHTML(_("Signature")) . ":&nbsp;</b>";
    $ante_body .= $my_ante;
    $ante_body .= "</td></tr><tr><td>";
  } else { $ante_body = ""; }
  return($ante_body);
}

/*********************************************************************/
/**
 * function gpg_strip_sign
 *
 * This function strips a gpg signature from the text
 *
 * @param string $signedtext
 * @return string unsigned body
 */
function gpg_strip_sign($signedtext) {
    $sep = "-----BEGIN PGP SIGNED MESSAGE-----";
    $sig = "-----BEGIN PGP SIGNATURE-----";
    $end = "-----END PGP SIGNATURE-----";
    if (!(strpos($signedtext,$sep) == 0)) { return $signedtext; }
    $exploded = explode("\n",$signedtext);
    $startbody=true;
    $skipnextline=false;
    foreach ($exploded as $line) {
        if ($line == $sep) {
            $startbody=true;
            $skipnextline=true;
            continue;
        }
        if ($line == $sig) {
            $startbody=false;
            continue;
        }
        if ($line == $end) {
            $startbody=true;
            continue;
        }
        if ($startbody && !$skipnextline) {
            $return = $return . $line . "\n";
            continue;
        } else { $skipnextline=false; continue; }
    }
    return $return;
}

/*********************************************************************/
/**
 * function gpg_check_sign_pgp_mime
 *
 * Used to read a detached signature file of type applicatiion/pgp-signature
 *
 * This function is called by the read_body hook (above)
 *
 * @param object $message  Message object as retrieves by sqimap__get_message
 * @param string $fullbodytext Body text string as retrieved directly from IMAP
 * @return integer $return
 */
function gpg_check_sign_pgp_mime($message,$fullbodytext) {
    $return=1;
    // attempt to check the signature if it looks like it has one
    global $data_dir;
    global $safe_data_dir;
    global $username;
    global $imapConnection, $passed_id;
    $safe_data_dir=getHashedDir($username,$data_dir) . DIRECTORY_SEPARATOR;

    $gpg_key_dir ="$safe_data_dir$username.gnupg";

    $messageData = &$message->entities[0];
    $messageSignature = &$message->entities[1];

    gpg_get_message_body_imap( $messageData, $imapConnection, $passed_id );
    gpg_get_message_body_imap( $messageSignature, $imapConnection, $passed_id );

    // PGP/MIME messages have the signature signed over the MIME headers as well
    $mimeHeader = gpg_get_mime_header( $messageData, $imapConnection, $passed_id );

    // the signed data is the contents between the mime boundary
    // RFC3156 section 5
    //get regular expression

    preg_match("/^.*boundary=\"?(.+(?=\")|.+).*/im",$fullbodytext,$reg);
    $regstr = '--'. $reg[1];
    $begin = strpos($fullbodytext,$regstr);

    //begin after /r/n
    $begin = $begin + strlen($regstr) + 2;

    //end before final /r/n
    $end = strpos($fullbodytext,$regstr,$begin) - 2;

    $messageSignedText = substr($fullbodytext,$begin,$end-$begin);
/*
commented until errors can be worked out
    if ($messageData->header->encoding == 'quoted-printable') {
    $messageSignedText = quoted_printable_decode($messageSignedText);
    }
*/
    // THIS IS VERY IMPORTANT. Must make sure all the lines end in \r\n
    // see RFC3156 section 5
    $messageSignedText = ereg_replace("\r\n", "\n", $messageSignedText );
    $messageSignedText = ereg_replace("\r", "\n", $messageSignedText );
    $messageSignedText = ereg_replace("\n", "\r\n", $messageSignedText );

    //$messageSignedText = escapeshellarg($messageSignedText);
    $messageSignedText = ereg_replace("\"", "\\\"", $messageSignedText );
    $ante_body = "";

    // first, we put the detached signature in a file
    // then we send the signed data as STDIN and tell GnuPG to get the
    // signature from the file
    $detachedSignatureFilename = tempnam( '', "GPG" );
    if( $detachedSignatureFilename != "FALSE" )
    {
        chmod( $detachedSignatureFilename, 0600 );

        $detachSigFile = fopen($detachedSignatureFilename,'w');
        fwrite( $detachSigFile, $messageSignature->decoded_body );
        fclose( $detachSigFile );

        // set this to 1 if you want to double check the prefs loading
        $debug=0;
        load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_system_defaults.txt',$debug);
        load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_local_prefs.txt',$debug);
        $path_to_gpg = $GLOBALS['GPG_SYSTEM_OPTIONS']['path_to_gpg'];

        $command = "echo -n \"$messageSignedText\" | $path_to_gpg --batch --no-tty --homedir $gpg_key_dir --verify ".$detachedSignatureFilename." - 2>&1";
        exec($command, $results, $returnval);
	$return=gpg_parse_output($results);
	$results = $return['signature'];
	$ante_body = '</td></tr><tr height="1%"><td align="left" class="nav">';
        $ante_body .= "<b>" . codeHTML(_("Signature")) . ":&nbsp;</b>";

        if (is_array($results)) {
            foreach ($results as $result) {
                $ante_body .= htmlspecialchars($result);
                $ante_body .= "<br>";
            }
        } else { $ante_body .= codeHTML(_("Unsigned")); }

        // delete our temp file
        unlink( $detachedSignatureFilename );
	$ante_body .= "</td></tr><tr><td>";
    }

    return ($ante_body);
}

/*********************************************************************/
/**
 * function gpg_compose_send_do
 *
 * This function is called by the main SM plugin_init (above)
 * to initialize our compose_send hook functions
 *
 * This is where the Encrypt on Send and
 * Encrypt and Sign on Send functions go.
 *
 * @return void
 */
/*********************************************************************/
function gpg_compose_send_do (&$composeMessage) {
    global $version,$base_uri;
    global $debug;
    global $session;
    global $compose_messages;
    global $data_dir;
    global $username;
    global $encrypt,$gpgsign,$passphrase,$encrypt_on_send;

    $debug=0;

	require_once(SM_PATH . 'plugins/gpg/gpg_functions.php');
	load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_system_defaults.txt',$debug);
	load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_local_prefs.txt',$debug);

	$cache_passphrase   = getPref ($data_dir, $username, 'cache_passphrase');
	$allowpassphrasecaching   = $GLOBALS['GPG_SYSTEM_OPTIONS']['allowpassphrasecaching'];

	sqgetGlobalVar('encrypt_on_send',$encrypt_on_send,SQ_POST);
	sqgetGlobalVar('sign_on_send',$sign_on_send,SQ_POST);
	sqgetGlobalVar('passphrase',$passphrase,SQ_POST);

	 //we need to pull these in as globals from SM for scoping reasons.
	
	sqgetGlobalVar('username',$username);
	sqgetGlobalVar('data_dir',$data_dir);

	if ($encrypt_on_send=='encrypt' || $sign_on_send=='sign'){
	    if ($encrypt_on_send=='encrypt'){
	        $encrypt = 'true';
	    }
	    if ($sign_on_send=='sign') {
		$gpgsign = 'true';
	    }

	    if (!$debug) {
		ob_start();
	    }
	    require_once(SM_PATH . 'plugins/gpg/gpg_sign_functions.php');
	    $messageAttachments = $composeMessage->getAttachments();
	    $checkattachment=strcmp(strtolower($messageAttachments[1]->mime_header->disposition->name),'attachment');
	    if ($gpgsign == 'true' && $encrypt!='true') {
		//get the passphrase
		if ($passphrase != '') {
		    if ($passphrase=='true' && gpg_is_passphrase_cached()) {
			$passphrase=gpg_get_cached_passphrase();
		    }
		}

		if (!$checkattachment) {
		    $return = gpg_sign_message($composeMessage->entities[0]->body_part,$passphrase,$debug);
		    if (!$return['errors'][0]) {
			$cyphertext = $return['cyphertext'];
			$composeMessage->entities[0]->body_part = $cyphertext;
		    } else {
			$sign_error['errors'] = $return['errors'];
			$sign_error['warnings'] = $return['warnings'];
			sqsession_register($sign_error,'encrypt_error');
			$cyphertext = '';
		    }
		    foreach($messageAttachments as $key=>$attachment) {
			if ($key > 0) {
			    $signmessage =& $composeMessage->entities[$key];
			    $signmessage = new Message();
			    $signmessage->entities[0] = $attachment;
			    if ($signmessage->entities[0]->mime_header->type0=='text') {
				$signmessage->entities[0]->mime_header->type0='application';
				$signmessage->entities[0]->mime_header->type1='octet-stream';
			    }
			    $signmessage->mime_header= new MessageHeader();
			    $signmessage->mime_header->type0='multipart';
			    $signmessage->mime_header->type1='signed';
			    $signmessage->mime_header->parameters['protocol'] = 'application/pgp-signature';
			    $signmessage->entities[1]= new Message();
			    $signmessage->entities[1]->mime_header = new MessageHeader();
			    $signmessage->entities[1]->mime_header->type0='application';
			    $signmessage->entities[1]->mime_header->type1='pgp-signature';
			    $filename = $attachment->att_local_name;
			    $ret = gpg_sign_attachment($filename, $passphrase);
			    $filename = $attachment->mime_header->disposition->properties['filename'];
			    $signmessage->entities[1]->att_local_name=$ret['filename'];
			    $signmessage->entities[1]->mime_header->parameters['name'] = $filename . ".sig.asc";
			    $signmessage->entities[1]->mime_header->disposition = new Disposition('attachment');
			    $signmessage->entities[1]->mime_header->disposition->properties['filename']=$filename . ".sig.asc";
		        }
		    }
		    $composeMessage->setEntIds($composeMessage);
		} else {
		    $return = gpg_sign_message($composeMessage->body_part,$passphrase,$debug);
		    if (!$return['errors'][0]) {
			$cyphertext = $return['cyphertext'];
			$composeMessage->body_part = $cyphertext;
		    } else {
			$sign_error['errors'] = $return['errors'];
			$sign_error['warnings'] = $return['warnings'];
			sqsession_register($sign_error,'encrypt_error');
			//print errors here sometime
			$cyphertext = '';
		    }
		}
	    } else {
		//set $cyphertext to be the return from gpg_encrypt.php
		require_once(SM_PATH.'plugins/gpg/gpg_encrypt.php');
		$cyphertext = gpg_overlook_encrypt($composeMessage);
	    }
	    /*if (!ctype_print($cyphertext)) {
    		sqgetGlobalVar('encrypt_error', $encrypt_error, SQ_SESSION);
		$string = _("Problems with the cyphertext output:").'\n';
		$string .= $cyphertext;
    		$encrypt_error['errors'][] = $string;
		sqsession_register($encrypt_error , 'encrypt_error');
		$cyphertext = '';
	    }*/

	    if (!$debug) {
		ob_end_clean();
	    } 

	    if (!$cyphertext && $checkattachment) {
		require_once(SM_PATH.'plugins/gpg/gpg_encrypt.php');
		//send us to gpg_encrypt.php again with the error array set so we can display errors.
		gpg_encrypt_on_send_error();		
		return false;
	    } else if (!$checkattachment) {
		$composeMessage->body_part = '';
	    } else {
		$composeMessage->body_part = $cyphertext;
	    }
	}; //end check for execute of 'on send' functionality

    return $composeMessage;
}; //end compose_send function

function gpg_decrypt_body_do($body) {
    global $gpg_key_dir, $path_to_gpg;
    global $username;
    global $data_dir;
    global $safe_data_dir;
    // il body non e' cryptato
    $sep = '-----BEGIN PGP MESSAGE-----';
    if (preg_match("/$sep/",$body) == false) { 
	sqsession_register('','decrypt_info_message');
    	return $body;
    }
    // il body e' cryptato ma non ho cliccato sul lucchetto
    sqgetGlobalVar('decrypt_do', $decrypt_do, SQ_POST);
    if ($decrypt_do != 1) {
	sqsession_register('','decrypt_info_message');
    	return $body;
    }
    sqgetGlobalVar('passphrase',  $passphrase, SQ_POST);
    
    if (!isset($gpg_key_dir)) {
        load_prefs_from_file(SM_PATH . 'plugins/gpg/gpg_system_defaults.txt',$debug);
        load_prefs_from_file(SM_PATH . 'plugins/gpg/gpg_local_prefs.txt',$debug);
        $path_to_gpg=($GLOBALS['GPG_SYSTEM_OPTIONS']['path_to_gpg']);
        $gpg_key_dir=($GLOBALS['GPG_SYSTEM_OPTIONS']['gpg_key_dir']);
        $gpg_key_dir ="$safe_data_dir$username.gnupg";
    }
    $debug = 0;
    $set_cached_passphrase=false;
    if ($passphrase != '') {
	if ($passphrase == 'true' and gpg_is_passphrase_cached()) {
    	    $passphrase=gpg_get_cached_passphrase();
	} else {
    	    if (gpg_is_passphrase_cacheable()) {
        	$set_cached_passphrase=true;
    	    }
	}
    }
    if (!$passphrase) {
	$return['errors'][] = _("You should insert a passphrase!");
	sqsession_register($return , 'decrypt_info_message');
	return $body;
    }
    $return = gpg_decrypt($debug, $body, $passphrase);
    if(is_array($return)) {
	sqsession_register($return , 'decrypt_info_message');
    }
    if ($return['plaintext']) { return $return['plaintext']; } else { return ''; }
} 

/*********************************************************************/
/**
 * function gpg_optpage_register_block_do ()
 *
 * This function formats and adds the plugin and its description to the
 * Options screen.
 *
 * @return void
 */
function gpg_optpage_register_block_do() {
  global $optpage_blocks, $base_uri;
  /**
   * soupNazi checks if this browser is capable of using the plugin.
   */
  if (!soupNazi()) {
    /**
     * The browser checks out.
     * Register Gpg with the $optionpages array.
     */
    $optpage_blocks[] = array(
         'name' => _("GPG/PGP"),
         'div'  => "div2",
         'url'  => $base_uri.'plugins/gpg/gpg_options.php',
         'desc' => _("The GPG Encryption Plugin will allow you to encrypt, sign, and decrypt messages in accordance with the OpenPGP standard for email security and authentication."),
         'js'   => false
    );
  }
}

function gpg_draw_compose_div() {
    echo '<div id="div_encr_sign" style="position:absolute;z-index:16;visibility:hidden;top:0px;left:0px;">
            <table class="optiontip" width="300pt" border=0 cellspacing=1 cellpadding=1>
                <tr>
                    <td aling="left" valign="top">
                        '.codeHTML(_("This message will be signed during transmission.")).'
                    </td>
                </tr>
            </table>
        </div>';
    echo '<div id="div_encr_msg" style="position:absolute;z-index:16;visibility:hidden;top:0px;left:0px;">
            <table class="optiontip" width="300pt" border=0 cellspacing=1 cellpadding=1>
                <tr>
                    <td aling="left" valign="top">
                        '.codeHTML(_("This message will be encrypted during transmission.")).'
                    </td>
                </tr>
            </table>
        </div>';
}

/*********************************************************************/
/**
 * function gpg_compose_row_do ()
 *
 * This function adds a "Encrypt Now" and "GPG Sign" link to the
 * "Compose" row during message composition.
 *
 * @return void
 */
function gpg_compose_row_do() {
  /**
   * Check if this browser is capable of displaying Gpg
   * correctly.
   */
    if (!soupNazi()) {

	/**
	 * Check to see if the user has a key directory (and presumably keys)
        * before displaying the "Encrypt Now" button.
        * Assume that if the user has a .gnupg directory, they have a keyring.
        * This assumption is not foolproof, but should work
        * Extend this to include signing check once secret keys are supported.
        */
        global $data_dir;
        global $username;
        global $base_uri;
        global $version;
        global $mailbox;
        global $body;
        global $safe_data_dir;
	global $sign_on_send,$encrypt_on_send;
        global $debug;
        $safe_data_dir = getHashedDir($username,$data_dir) . DIRECTORY_SEPARATOR;
        if (isset($_SESSION['gpg_dbody']) and isset($_GET['gpgreply'])) {
    	    $body=$_SESSION['gpg_dbody'];
        }
        unset($_SESSION['gpg_dbody']);
        if (substr_count($mailbox, "Drafts")>0) {
    	    $body = gpg_strip_sign($body);
        }
        $gpg_key_dir ="$safe_data_dir$username.gnupg";
        $pubring="$gpg_key_dir/pubring.gpg";
	$secring="$gpg_key_dir/secring.gpg";
        global $no_encrypt_on_setup;
        global $gpg_export;
        $use_signing_key_id = getPref ($data_dir, $username, 'use_signing_key_id');
	$auto_encrypt       = getPref ($data_dir, $username, 'auto_encrypt');
        $auto_sign          = getPref ($data_dir, $username, 'auto_sign');
        $no_signing_passwd  = getPref ($data_dir, $username, 'no_signing_passwd');
        $cache_passphrase   = getPref ($data_dir, $username, 'cache_passphrase');
        $allowpassphrasecaching   = $GLOBALS['GPG_SYSTEM_OPTIONS']['allowpassphrasecaching'];

        $passphrase_cached = 'false';
	if (gpg_get_cached_passphrase() != 'false') {
    	    $passphrase_cached = 'true';
	};

        if ($auto_encrypt == 'true' || $encrypt_on_send=='encrypt') { $aechecked = true; } else { $aechecked = false; }
        if ($auto_sign == 'true' || $sign_on_send=='sign') { $aschecked = true; } else { $aschecked = false; }

	//$debug=0;

	//load our system and local prefs files so we can figure out where things are
	load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_system_defaults.txt',$debug);
	load_prefs_from_file(SM_PATH.'plugins/gpg/gpg_local_prefs.txt',$debug);

	$debug=$GLOBALS['GPG_SYSTEM_OPTIONS']['debug'];

	if ($debug) { echo "<p>no_encrypt_on_setup: $no_encrypt_on_setup<br>gpg_export: $gpg_export<br>pubring: $pubring<br>secring: $secring<br>"; }

	/**
	 * Some people may choose to disable javascript even though their
	 * browser is capable of using it. So these folks don't complain,
	 * use document.write() so the "Encrypt Now" and "GPG Sign"
	 * buttons are not displayed if js is off in the browser.
	 *
	 * Also, check to make sure that the user has keyrings on the system before
	 * displaying buttons for features they can't use.
	 *
	 * $no_encrypt_on_setup is a variable that is set to make sure that the
	 * buttons don't display after we have already encrypted the message.
	 * there is no point in displaying them after successful encryption.
	 */
	if ($chdir_first) { chdir ('../'); };

	$ok_row = 0;
	$ok_sign = 0;
	$ok_msg = 0;
	// add the "GPG Sign" Checkbutton
	if (file_exists($secring) && filesize($secring)>0 
	    && $GLOBALS['GPG_SYSTEM_OPTIONS']['allowprivatekeys']=='true' 
	    && $use_signing_key_id=='true' && !$no_encrypt_on_setup) {
	    $ok_row = 1;
	    $ok_sign = 1;
	}
	// add the "GPG message encrypt" Checkbutton
	if (file_exists($pubring) && filesize($pubring)>0 && !$no_encrypt_on_setup) {
	    $ok_row = 1;
	    $ok_msg = 1;
	}
	if ($ok_row) {
	    echo '<script type="text/javascript">
		function gpg_composeSubmit(objform) {
		    if (objform.passphrase.value!="true") {
			if (objform.sign_on_send.checked && sendClicked) {
			    new_win_overlook("Secure_GPG_Signing","'.$base_uri.'plugins/gpg/gpg_pop_init.php?MOD=passpop&psaction=sign&addbasepath='.$base_uri.'",300,200,1,200,200);
    			    return false;
			} else {
			    if (objform.sign_on_send.checked && draftClicked) {
				new_win_overlook("Secure_GPG_Signing","'.$base_uri.'plugins/gpg/gpg_pop_init.php?MOD=passpop&psaction=signdraft&addbasepath='.$base_uri.'",300,200,1,200,200);
    				return false;
			    }
			}
		    }
		    return true;
		}
	    </script>';
	    echo addHidden('gpgsign','');
	    echo addHidden('passphrase',$passphrase_cached);
	    echo '<tr>';
		echo '<td class="nav" align="left" width="10%">'.codeHTML(_("Pgp/GPG")).':</td>';
		echo '<td class="nav" align="left" width="90%">';
	}
	if ($ok_sign) {
	    echo addCheckBox('sign_on_send', $aschecked, 'sign', 'onMouseOver="showtip(event,\'div_encr_sign\')" onMouseOut="hidetip(\'div_encr_sign\')"');
	    echo "&nbsp;".codeHTML(_("Sign this message"))."&nbsp;&nbsp;";
    	}
	if ($ok_msg) {
	    echo addCheckBox('encrypt_on_send', $aechecked, 'encrypt','onMouseOver="showtip(event,\'div_encr_msg\')" onMouseOut="hidetip(\'div_encr_msg\')"');
	    echo "&nbsp;".codeHTML(_("Encrypt message"));
	}
	if ($ok_row) {
	    echo '</td>';
	    echo '</tr>'."\n";
	}
    }; //end soupNazi check
}; //end compose_row function

/*********************************************************************/
/**
 *
 * $Log: gpg_hook_functions.php,v $
 * Revision 1.62  2003/12/19 21:18:05  ke
 * -added debug output to update_trustdb command
 *
 * Revision 1.61  2003/12/19 20:44:59  ke
 * -changed trustdb update code to use centralized gpg_update_trustdb function
 * -only update trustdb when untrusted keys are found
 *
 * Revision 1.60  2003/12/18 22:18:05  ke
 * -added a trustdb check before confirming a clearsigned signature
 *
 * Revision 1.59  2003/12/18 19:44:14  ke
 * -changed to no longer use data_dir in system keyring path
 *
 * Revision 1.58  2003/12/16 20:00:03  brian
 * changed instances of $system_keyring_file = $safe_data_dir to
 *    $system_keyring_file = $data_dir becasue $safe_data_dir is a
 *    hashed dir on a per-user basis
 *
 * Revision 1.57  2003/12/16 19:28:45  brian
 * fixed indents in the gpg_handle_signature_do fn
 *
 * Revision 1.56  2003/12/16 19:27:07  brian
 * fixed indents in the gpg_handle_signature_do fn
 *
 * Revision 1.55  2003/12/11 20:42:32  ke
 * -added $extra_cmd to the command string, to include the system keyring
 *
 * Revision 1.54  2003/12/11 19:56:17  brian
 * added shared corporate keyring support to plainsigned verification
 *
 * Revision 1.53  2003/12/08 21:10:36  ke
 * -added translation section to options block text
 *
 * Revision 1.52  2003/12/02 15:26:14  ke
 * -added check for definition of base_uri, reset to get_location . SM_PATH if not set
 *
 * Revision 1.51  2003/12/02 04:11:53  ke
 * -added error check on signature data and display error message if no verification can be done on an attachment
 *
 * Revision 1.50  2003/12/02 03:18:20  ke
 * -changed to check for a body in the message object before downloading the full imap body
 *
 * Revision 1.49  2003/12/02 02:59:13  ke
 * -added calls to gpg_parse_output for signature lines on signature attachments
 * -removed text stripping code which is already done in gpg_parse_output
 * -added htmlspecialchars call to see email address in <>'s
 *
 * Revision 1.48  2003/11/25 21:54:59  ke
 * -translated Signature field header in read_body hook
 * -changed check_sign function to use gpg_parse_output to trap errors
 * -added htmlspecialchars call to show email address in signature information
 * bug 113
 *
 * Revision 1.47  2003/11/25 02:17:30  ke
 * -added missing getHashedDir definition for safe_data_dir in compose_row function
 *
 * Revision 1.46  2003/11/25 02:04:02  ke
 * -added safe_data_dir to code accessing the keyring files
 * bug 116
 *
 * Revision 1.45  2003/11/20 17:36:07  ke
 * -removed text binding code at the end of decrypt_link function, to allow it to be called from elsewhere
 *
 * Revision 1.44  2003/11/18 19:44:33  ke
 * -removed autosetting locale to gpg when requiring gpg_hook_functions.php
 *
 * Revision 1.43  2003/11/18 19:33:21  ke
 * -removed auto-setting text domain to squirrelmail at end of file
 * -this allows gpg_hook_functions to be included in other places without breaking localization strings
 *
 * Revision 1.42  2003/11/17 19:24:41  ke
 * -added check for quoted-printable in the hook where the body is grabbed from the imap server
 * -added code to strip out encryped message in the body, store in a session variable
 * -changed window.open and form action to use SM_PATH
 * bug 105
 *
 * Revision 1.41  2003/11/13 19:34:31  ke
 * -added SM_PATH redirection to import key function to allow calling from anywhere
 * bug 105
 *
 * Revision 1.40  2003/11/13 18:30:11  ke
 * -added missing closed paren from my last commit
 *
 * Revision 1.39  2003/11/13 18:24:46  ke
 * -internationalized Decrypt Attachment statement
 * -added bindtextdomain calls to the attachment handlers
 *
 * Revision 1.38  2003/11/10 19:53:51  ke
 * -added bindtextdomain calls within hook functions, for translation purposes
 *
 * Revision 1.37  2003/11/07 22:09:14  ke
 * -added dlfilename being passed to decrypt_attachment php file
 *
 * Revision 1.36  2003/11/07 19:53:18  ke
 * -added encryption check for signature header
 *
 * Revision 1.35  2003/11/07 18:46:56  ke
 * -fixed bug in confirmation of attached signatures function
 * -added Signature: header for signed message output
 *
 * Revision 1.34  2003/11/07 16:42:50  ke
 * -added numerous debug statements in signing and downloading attachment code
 * -committing brian's SM_PATH additions in different hook functions
 * -now sets any text files being signed to be application/octet-stream for proper encoding
 *
 * Revision 1.33  2003/11/04 21:38:40  brian
 * change to use SM_PATH
 *
 * Revision 1.32  2003/11/03 18:41:32  ke
 * -added startMessage and sort variables submitted to gpgdecrypt
 *
 * Revision 1.31  2003/11/03 17:32:59  ke
 * -changed to use abstracted checking functions before retreiving cached passphrase
 * bug 66
 *
 * Revision 1.30  2003/11/01 22:01:26  brian
 * infrastructure changes to support removal of MakePage functions
 *
 * Revision 1.29  2003/10/30 19:40:20  brian
 * modified all localized strings to use double quotes.
 * Bug 35
 *
 * Revision 1.28  2003/10/27 19:31:01  ke
 * -added urlpath to pass to popup, includes proper path to submit
 * bug 82
 *
 * Revision 1.27  2003/10/27 18:19:34  ke
 * -added functions to verify detached signatures
 * -added function to download attachment to a tempfile
 *
 * Revision 1.26  2003/10/20 22:54:05  ke
 * -added compose hook for sending detached signatures for attachments
 * -had to patch squirrelmail to handle these, hopefully to be committed soon
 *
 * Revision 1.25  2003/10/17 13:12:05  brian
 * corrected phpdoc warnings after updates
 *
 * Revision 1.24  2003/10/17 12:50:20  brian
 * added package and author tags
 *
 * Revision 1.23  2003/10/16 20:36:49  ke
 * -fixed bug when signing a message with an attachment
 * -no signing of attachments yet, but at least the body will be signed.
 *
 * Revision 1.22  2003/10/16 20:21:48  ke
 * -added variable addbasepath to smooth operation when errors cause compose.php to be re-included
 * -fixes encryption and signing buttons
 * -does NOT fix popups
 * bug 82
 *
 * Revision 1.21  2003/10/16 13:55:00  brian
 * improved function headers
 *
 * Revision 1.20  2003/10/15 19:22:14  ke
 * -added code to verify detached signatures using all contents between mime boundaries
 * bug 33
 *
 * Revision 1.19  2003/10/13 22:25:46  brian
 * - added checks for secring and allowprivatekeys to gpg_decrypt_link fn
 * - display errors if those test fail rather than the decrypt button.
 * Bug 85
 *
 * Revision 1.18  2003/10/11 21:33:38  ke
 * -Added javascript to allow Save Draft to prompt for passphrase
 * -Added gpg_strip_sign function to return body of signed text
 * -Use above function to show only body when returning to compose from a draft
 * bug 77
 *
 * Revision 1.17  2003/10/07 22:22:10  ke
 * -changed javascript in compose hooks to call consolidated passphrase interface
 * bug 13
 *
 * Revision 1.16  2003/10/07 18:34:35  ke
 * -Added error handling within sign on send code
 *
 * Revision 1.15  2003/10/07 17:10:39  brian
 * added calls to get next and prev message ids to gpg_decrypt_link fn
 * Bug 65
 *
 * Revision 1.14  2003/10/06 22:51:48  ke
 * -changed import_link function to import keys direction into keyring_main.php
 * -hides any keys found in the body in a hidden variable, posts directly to keyring_main
 *
 * Revision 1.13  2003/10/03 22:43:34  ke
 * -Removed set_cached_passphrase call from sign_on_send handling of compose_send hook
 * -Passphrase is cached in cachepass.mod before this is ever called
 * -All compose.php buttons work properly, no longer pop up passphrase request
 *
 * Revision 1.12  2003/10/01 23:09:42  brian
 * added support for reply from decrypt
 * - extra variables passed in gpg_decrypt_link fn
 * - gpg_dbody flag processed in gpg_compose_row fn
 * Bug 65
 *
 * Revision 1.11  2003/09/30 01:23:11  ke
 * -Added internationalization to all messages and buttons in gpg_hook_functions.
 * -Ignored the titles of popup windows for now, maybe we'll want to work on those later?
 * bug 35
 *
 * Revision 1.10  2003/09/29 16:15:01  brian
 * added check for system and user prefs for passphrase cachign to functions:
 * - gpg_decrypt_link
 * - gpg_compose_row_do
 * - gpg_compose_send_do
 * Bug 40
 *
 * Revision 1.9  2003/09/26 19:24:50  brian
 * adjust javascript to actually work
 * Bug 40
 *
 * Revision 1.8  2003/09/26 15:42:13  brian
 * Changes to gpg_decrypt_link fn to support caching:
 * - add a form and scripting to control action
 * Bug 40
 *
 * Revision 1.7  2003/09/26 02:31:56  ke
 * -Added gpg_functions.php requirement for caching
 * -Removed unneccessary javaascript from compose_row_do
 * -Changed operation of compose_row_do to use popups or submit depending on status of cached passphrase
 *
 * Revision 1.6  2003/09/23 22:10:38  ke
 * Changed redirection Location: header sent to use global $base_uri
 * This allows the redirection to operate properly when called from different locations.
 *
 * Revision 1.5  2003/09/20 01:33:33  brian
 * moved body_part 'stuffing to gpg_encrypt.php
 *
 * Revision 1.4  2003/09/18 22:14:32  ke
 * -Added javascript and interface changes for sign-on-send
 * -Moved encrypt-on-send checkbox to appear after signing widgets
 * -Completed sign-on-send, encrypt-on-send and combinations functionality
 * Bug 55
 *
 * Revision 1.3  2003/09/17 18:22:36  ke
 * -Altered functional calls for compose_send_do, ensured no name collision with message object
 * -part of the encrypt on send fix
 * Bug 53
 *
 * Revision 1.2  2003/09/16 00:03:28  ke
 * Added _do to end of hook function names, to allow wrapper functions to call them
 *
 * Revision 1.1  2003/09/15 23:18:11  ke
 * Newly created hook functions file
 * -Split to speed calls to setup.php
 *
 * Revision 1.50  2003/07/14 15:34:06  brian
 * changed main options block to title: Encryption Options (GPG/OpenPGP)
 *
 * Revision 1.49  2003/07/09 13:53:58  brian
 * added processing of $gpg_export flag to change compose page target
 * Bug 41
 *
 * Revision 1.48  2003/07/07 20:33:58  brian
 * - modified code to support import from email
 * - added gpg_import_link fn
 * Bug 46
 *
 * Revision 1.47  2003/07/02 22:45:42  brian
 * pulled out the 'Encrypt&Sign on Send' code entirely for now.
 *
 * Revision 1.46  2003/07/02 22:39:53  brian
 * moved closing coment bracket
 *
 * Revision 1.45  2003/07/02 22:36:54  brian
 * comment out encrypt&sign on send, since it doesn't work anyway
 *
 * Revision 1.44  2003/07/02 22:28:36  brian
 * fixed inadvertent bug that broke Encrypt&Sign
 *
 * Revision 1.43  2003/05/28 19:17:12  brian
 * modified to fix non-working encryptcapability (broken during work on encrypt&sign on send)
 *
 * Revision 1.42  2003/05/19 17:51:05  brian
 * added 'global $version' to compose_row fn
 *
 * Revision 1.41  2003/05/16 16:29:15  brian
 * added version check to disable encrypt on send
 * (in compose_body_row hook) in SM < 1.4.0
 *
 * Revision 1.40  2003/05/16 16:20:11  brian
 * added version check to disable encrypt on send (in compose_end hook) in SM < 1.4.0
 *
 * Revision 1.39  2003/05/16 13:44:39  brian
 * added tags for phpdoc
 *
 * Revision 1.38  2003/05/15 22:47:21  brian
 * - improved handlnig in compose_send
 * - added $debug flags
 * - added comments to explain use of globals
 *
 * Revision 1.37  2003/05/14 01:34:24  vinay
 * *** empty log message ***
 *
 * Revision 1.36  2003/05/13 22:52:23  brian
 * added compose_send hook functionality for encrypt on send
 * Bug 26
 *
 * Revision 1.35  2003/05/10 12:23:59  brian
 * added function header blocks to functions that were missing them
 *
 * Revision 1.34  2003/05/02 15:37:06  brian
 * checked in Ryan's code for detached signatures after minor mods
 * Bug 33
 *
 * Revision 1.33  2003/04/16 02:33:32  brian
 * added 'Encrypt&Sign' button
 * Bug 11
 *
 * Revision 1.32  2003/04/09 12:59:20  brian
 * single quote values in 'Encrypt Now' button
 *
 * Revision 1.31  2003/04/04 03:05:49  brian
 * added global $passed_id, $mailbox back into decrypt_link function -- needed for document.write
 *
 * Revision 1.30  2003/04/04 01:56:11  tyler
 * - added $uid_support to the sqimap_run_command query
 *
 * Revision 1.29  2003/04/04 00:09:48  brian
 * changed so that fetch_body function is called only once, so we only connect once to the imap stream
 *
 * Revision 1.28  2003/04/03 23:45:35  brian
 * changed check_sign and decrypt_link to use the standard SM core command of sqimap_run_command, rather that sqimap_read_data, becasue read_data didn't work the same under SM 1.4
 *
 * Revision 1.27  2003/04/03 02:32:22  brian
 * Decoupled signature verification and display of the decryption button.
 *
 * Revision 1.26  2003/04/02 12:25:42  brian
 * modified window.open function for decrypt to:
 * - make passphrase dialog smaller
 * - allow scrollbars
 *
 * Revision 1.25  2003/04/01 07:58:18  brian
 * corrected document .write link to work in SM 1.4
 *
 * Revision 1.24  2003/03/31 22:02:45  brian
 * modified _GET parameters for decrypt now button
 *
 * Revision 1.23  2003/03/31 21:57:31  brian
 * modified to add passed_value and mailbox to submit of decrypt now button
 *
 * Revision 1.22  2003/03/31 15:18:25  brian
 * modified gpg_decrypt_link function to use document,.write and window.open functions to create popup.
 *
 * Revision 1.21  2003/03/31 15:03:41  brian
 * - modified to remnove double declaration of gpg_check_sign
 * - file now correctly declares gpg_decrypt_link
 *
 * Revision 1.20  2003/03/31 14:57:38  brian
 * - modified signing link to use new gpg_pop_init.php file
 * - added link for decrypt now
 * - placed signing and decryption functions under read_body_header
 *   initialization function
 * Bug 8
 *
 * Revision 1.19  2003/03/25 21:43:23  brian
 * Bug 6
 * Slightly better handling of whether to display the buttons or not after encrypt.
 *
 * Revision 1.18  2003/03/17 18:55:41  brian
 * - progress towards SM v >=1.3.1 compatibility
 * - path selection for includes now works on both
 *   SM 1.2.x and SM >= 1.3.1
 *
 * Revision 1.17  2003/03/15 22:03:32  brian
 * moved strings.php include to outside of the SM version check
 * strings.php sets the SM version...
 *
 * Revision 1.16  2003/03/13 04:04:16  brian
 * modified GPG Sign button calling code.
 *
 * Revision 1.15  2003/03/12 15:00:16  brian
 * - added document action to change the document action after encrypting an email
 * - @todo make sure that it works even after unsuccessful submit
 * - @todo make syntax cross browser compatible
 *
 * Revision 1.14  2003/03/12 14:34:40  brian
 * - added function header comment blocks to all functions
 *
 * Revision 1.13  2003/03/12 05:02:58  tyler
 * - reduced the size of the message_sign popup window
 *
 * Revision 1.12  2003/03/12 01:43:58  tyler
 * - test for secring file now checks for zero length file
 *
 * Revision 1.11  2003/03/12 01:36:28  tyler
 * - Initial attempt at signature verification on read. New hook added.
 *
 * Revision 1.10  2003/03/11 21:25:18  tyler
 * - helps if you define $privatekeysallowed otherwise it's always 0 :)
 *
 * Revision 1.9  2003/03/11 19:22:08  tyler
 * - Modified to use $allowprivatekeys preference to decide if Sign button should be displayed
 *
 * Revision 1.8  2003/03/11 02:45:24  tyler
 * - modified code to only exclude encrypt now button after encryption routine
 *
 * Revision 1.7  2003/03/11 01:29:51  brian
 * fixed bug with pressing button twice by not showing buttons after the encryption routine has been run correctly
 *
 * Revision 1.6  2003/03/11 01:06:48  tyler
 * - renamed filename variable to pubring
 * - added secring variable
 * - rewrote the button building code to use document.write so folks with
 *   javascript turned off wont catch errors, buttons just wont get displayed
 * - rewrote the button building code to only display if ring file available
 * - converted 'Sign Now' button to 'GPG Sign' so as not to be confused with
 *   other "sign" buttons
 * - converted 'Encrypt Now' button to 'GPG Encrypt'
 * - converted code called by 'Sign Now' to pop a window
 *
 * Revision 1.5  2003/03/09 14:35:54  brian
 * Added Tyler's "Sign Now" button
 * todo - only show button if user has a secret key
 *
 * Revision 1.4  2003/03/06 23:42:48  brian
 * Added check for SM ver > 1.3
 *
 * Revision 1.3  2003/02/26 17:07:55  brian
 * Added check so that the Encrypt Now button will only be displayed if the user has a keyring directory.
 *
 * Revision 1.2  2002/12/05 19:25:46  brian
 * Added ID and Log tags
 *
 *
 */

?>
