<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * search.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * $Id: search.php,v 1.3 2004/02/25 15:02:08 marco Exp $
 */

/* Path for SquirrelMail required files. */
define('SM_PATH','../');

/* SquirrelMail required files. */
require_once(SM_PATH . 'include/validate.php');
require_once(SM_PATH . 'functions/imap.php');
require_once(SM_PATH . 'functions/imap_search.php');
require_once(SM_PATH . 'functions/imap_mailbox.php');
require_once(SM_PATH . 'functions/strings.php');
require_once(SM_PATH . 'functions/forms.php');
require_once(SM_PATH . 'functions/global.php');

global $allow_thread_sort, $custom_css;

/* get globals we may need */
sqgetGlobalVar('username', $username, SQ_SESSION);
sqgetGlobalVar('key', $key, SQ_COOKIE);
sqgetGlobalVar('delimiter', $delimiter, SQ_SESSION);
sqgetGlobalVar('onetimepad', $onetimepad, SQ_SESSION);
sqgetGlobalVar('base_uri', $base_uri, SQ_SESSION);

sqgetGlobalVar('mailbox', $mailbox);
if ($mailbox) {
    $patermailbox = $mailbox;	
    if ($patermailbox == 'INBOX') {
    	$showpatermailbox = _("INBOX");
    } else {
    	$showpatermailbox = imap_utf7_decode_local($patermailbox);
    	$a = explode($delimiter,$showpatermailbox);
    	$showpatermailbox = array_pop($a);
    }
}
sqgetGlobalVar('submitsearch', $submit);
sqgetGlobalVar('what', $what);
sqgetGlobalVar('where', $where);
sqgetGlobalVar('subdir', $subdir);
sqgetGlobalVar('count', $count);
/* end of get globals */

/*  function to get the recent searches and put them in the attributes array  */
function get_recent($username, $data_dir) {
    $attributes = array();
    $types = array('search_what', 'search_where', 'search_folder');
    $recent_count = getPref($data_dir, $username, 'search_memory', 0);
    for ($x=1;$x<=$recent_count;$x++) {
        reset($types);
        foreach ($types as $key) {
            $attributes[$key][$x] = getPref($data_dir, $username, $key.$x, "");
        }
    }
    return $attributes;
}

/*  function to get the saved searches and put them in the saved_attributes array  */
function get_saved($username, $data_dir) {
    $saved_attributes = array();
    $types = array('saved_what', 'saved_where', 'saved_folder');
    foreach ($types as $key) {
        for ($x=1;;$x++) {
            $saved_attributes[$key][$x] = getPref($data_dir, $username, $key."$x", "");
            if ($saved_attributes[$key][$x] == "") {
                array_pop($saved_attributes[$key]);
               break;
            }
        }
    }
    return $saved_attributes;
}

/*  function to update recent pref arrays  */
function update_recent($what, $where, $mailbox, $username, $data_dir) {
    $attributes = array();
    $types = array('search_what', 'search_where', 'search_folder');
    $input = array($what, $where, $mailbox);
    $attributes = get_recent( $username, $data_dir);
    reset($types);
    $dupe = 'no';
    for ($i=1;$i<=count($attributes['search_what']);$i++) {
        if (isset($attributes['search_what'][$i])) {
            if ($what == $attributes['search_what'][$i] &&
                $where == $attributes['search_where'][$i] &&
                $mailbox == $attributes['search_folder'][$i]) {
                    $dupe = 'yes';
            }
        }
    }
    if ($dupe == 'no') {
        $i = 0;
        foreach ($types as $key) {
            array_push ($attributes[$key], $input[$i]);
            array_shift ($attributes[$key]);
            $i++;
        }
        $recent_count = getPref($data_dir, $username, 'search_memory', 0);
        $n=0;
        for ($i=1;$i<=$recent_count;$i++) {
            reset($types);
            foreach ($types as $key) {
                setPref($data_dir, $username, $key.$i, $attributes[$key][$n]);
            }
            $n++;
        }
    }
}

/*  function to forget a recent search  */
function forget_recent($forget_index, $username, $data_dir) {
    $attributes = array();
    $types = array('search_what', 'search_where', 'search_folder');
    $attributes = get_recent( $username, $data_dir);
    reset($types);
    foreach ($types as $key) {
        array_splice($attributes[$key], $forget_index - 1, 1);
        array_unshift($attributes[$key], '');
    }
    reset($types);
    $recent_count = getPref($data_dir, $username, 'search_memory', 0);
    $n=0;
    for ($i=1;$i<=$recent_count;$i++) {
        reset($types);
        foreach ($types as $key) {
            setPref($data_dir, $username, $key.$i, $attributes[$key][$n]);
        }
        $n++;
    }
}

/*  function to delete a saved search  */
function delete_saved($delete_index, $username, $data_dir) {
    $types = array('saved_what', 'saved_where', 'saved_folder');
    $attributes = get_saved($username, $data_dir);
    foreach ($types as $key) {
        array_splice($attributes[$key], $delete_index, 1);
    }
    reset($types);
    $n=0;
    $saved_count = count($attributes['saved_what']);
    $last_element = $saved_count + 1;
    for ($i=1;$i<=$saved_count;$i++) {
        reset($types);
        foreach ($types as $key) {
            setPref($data_dir, $username, $key.$i, $attributes[$key][$n]);
        }
    	$n++;
    }
    reset($types);
    foreach($types as $key) {
	removePref($data_dir, $username, $key.$last_element);
    }
}

/*  function to save a search from recent to saved  */
function save_recent($save_index, $username, $data_dir) {
    $attributes = array();
    $types = array('search_what', 'search_where', 'search_folder');
    $saved_types = array(0 => 'saved_what', 1 => 'saved_where', 2 => 'saved_folder');
    $saved_array = get_saved($username, $data_dir);
    $save_index = $save_index -1;
    $saved_count = (count($saved_array['saved_what']) + 1);
    $attributes = get_recent ($username, $data_dir);
    $n = 0;
    foreach ($types as $key) {
        $slice = array_slice($attributes[$key], $save_index, 1);
        $name = $saved_types[$n];
        setPref($data_dir, $username, $name.$saved_count, $slice[0]);
        $n++;
    }
}

function printSearchMessages($msgs,$cnt, $imapConnection, $where, $what, $usecache = false, $newsort = false) {
    global $sort, $color, $base_uri, $where, $what, $subdir;
    if ($cnt > 0) {
	$msort = calc_msort($msgs, $sort);

        $msg_cnt_str = get_msgcnt_str(1, $cnt, $cnt);

	echo '<tr><td>'; 
	echo '<table align="center" class="titleright" valign="middle" width="100%" height="100%" cellpadding="0" cellspacing="0">'; 
	printHeaderSearch();
	echo '</table>';
	echo '</td></tr>';

	echo '<tr><td>';
	echo '<table align="center" class="border" valign="middle" width="100%" height="100%" cellpadding="0" cellspacing="0">'; 
	echo addForm($base_uri.'src/move_messages.php', 'POST', 'FormSearch');
	echo addHidden('overlook_action', '');
	echo addHidden('msg', '');
	echo addHidden('mailboxes', '');
	echo addHidden('mailbox', '');
	echo addHidden('targetMailbox', '');
	echo addHidden('location', $base_uri.'src/search.php?where='.$where.'&what='.$what.'&subdir='.$subdir.'&mailbox=');

        displayMessageArraySearch($imapConnection, $cnt, 1, 
		          $msort, $sort, $color, $cnt, $where, $what);
	echo '</form>';
	echo '</table>';
	echo '</td></tr>';
    }
}			      

/* Generic function to convert the msgs array into an HTML table. */
function displayMessageArraySearch($imapConnection, $num_msgs, $start_msg,
                             $msort, $sort, $color,
                             $show_num, $where=0, $what=0) {
    global $imapServerAddress, $use_mailbox_cache, $index_order,$base_uri,
           $indent_array, $thread_sort_messages, $allow_server_sort, $patermailbox,
           $server_sort_order, $PHP_SELF,$squirrelmail_plugin_hooks, $select_message_onclick,
	    $custom_css;

    $res = getEndMessage($start_msg, $show_num, $num_msgs);
    $start_msg = $res[0];
    $end_msg   = $res[1];

    /* get indent level for subject display */
    if ($thread_sort_messages == 1 && $num_msgs) {
        $indent_array = get_parent_level($imapConnection);
    }

    $real_startMessage = $start_msg;
    if ($sort == 6) {
        if ($end_msg - $start_msg < $show_num - 1) {
            $end_msg = $end_msg - $start_msg + 1;
            $start_msg = 1;
        } else if ($start_msg > $show_num) {
            $end_msg = $show_num;
            $start_msg = 1;
        }
    }
    $endVar = $end_msg + 1;

    /*
     * Loop through and display the info for each message.
     * ($t is used for the checkbox number)
     */
    $t = 0;

    if (!$num_msgs) {

	echo '&nbsp;</td></tr><tr><td align="center" colspan="'.count($index_order).'"><b>'.codeHTML(_("THIS FOLDER IS EMPTY")).'</b>';

    } elseif ($start_msg == $end_msg) {

    /* if there's only one message in the box, handle it differently. */
        if ($sort != 6) {
            $i = $start_msg;
        } else {
            $i = 1;
        }
        reset($msort);
        $k = 0;
        do {
            $key = key($msort);
            next($msort);
            $k++;
        } while (isset ($key) && ($k < $i));
    	    printMessageInfoSearch($imapConnection, $t, true, $key,
                         $real_startMessage, $where, $what,$num_msgs);
    } else {
        $i = $start_msg;
        reset($msort);
        $k = 0;
        do {
            $key = key($msort);
            next($msort);
            $k++;
        } while (isset ($key) && ($k < $i));
        $not_last = true;
        do {
            if (!$i || $i == $endVar-1) $not_last = false;
        	printMessageInfoSearch($imapConnection, $t, $not_last, $key,
                                 $real_startMessage, $where, $what, $num_msgs);
            $key = key($msort);
            $t++;
            $i++;
            next($msort);
        } while ($i && $i < $endVar);
    }
}


function printMessageInfoSearch($imapConnection, $t, $not_last=true, $key,
                          $start_msg, $where, $what, $num_msgs) {
    global 
           $color, $msgs, $msort, $td_str, $msg, $custom_css,
           $default_use_priority,$squirrelmail_plugin_hooks,
           $message_highlight_list,
           $index_order,
           $indent_array,   /* indent subject by */
           $pos,            /* Search postion (if any)  */
           $thread_sort_messages, /* thread sorting on/off */
           $server_sort_order, /* sort value when using server-sorting */
           $row_count, $base_uri, $max_char_obj_m,
           $allow_server_sort; /* enable/disable server-side sorting */
    $color_string = $color[4];

    if ($GLOBALS['alt_index_colors']) {
        if (!isset($row_count)) {
            $row_count = 0;
        }
        $row_count++;
        if ($row_count % 2) {
            if (!isset($color[12])) {
                $color[12] = '#EAEAEA';
            }
            $color_string = $color[12];
        }
    }
    $msg = $msgs[$key];

    if (handleAsSent($msg['folder_complete'])) {
       $msg['FROM'] = $msg['TO'];
    }
    $msg['FROM'] = parseAddress($msg['FROM'],1);
    
       /*
        * This is done in case you're looking into Sent folders,
        * because you can have multiple receivers.
        */

    $senderNames = $msg['FROM'];
    $senderName  = '';
    if (sizeof($senderNames)){
        foreach ($senderNames as $senderNames_part) {
            if ($senderName != '') {
                $senderName .= ', ';
            }
            if ($senderNames_part[1]) {
                $senderName .= $senderNames_part[1];
            } else {
                $senderName .= $senderNames_part[0];
            }
        }
    }

    $style="";

    if (isset($msg['FLAG_FLAGGED']) && ($msg['FLAG_FLAGGED'] == true)) {
	$style .= "color:$color[2];";
    }
    if (!isset($msg['FLAG_SEEN']) || ($msg['FLAG_SEEN'] == false)) {
	$style .= "font-weight:bold;";
    }
    if (isset($msg['FLAG_DELETED']) && $msg['FLAG_DELETED']) {
	$style .= "color:$color[9];";
    }
    if ($style) { $style = "style='$style'"; }

    if ($where && $what) {
        $searchstr = '&where='.$where.'&what='.$what;
    } else {
        $searchstr = '';
    }

    if (is_array($message_highlight_list) && count($message_highlight_list)) {
        $msg['TO'] = parseAddress($msg['TO']);
        $msg['CC'] = parseAddress($msg['CC']);
        foreach ($message_highlight_list as $message_highlight_list_part) {
            if (trim($message_highlight_list_part['value']) != '') {
                $high_val   = strtolower($message_highlight_list_part['value']);
                $match_type = strtoupper($message_highlight_list_part['match_type']);
                if($match_type == 'TO_CC') {
                    $match = array('TO', 'CC');
                } else {
                    $match = array($match_type);
                }
                foreach($match as $match_type) {
                    switch($match_type) {
                        case('TO'):
                        case('CC'):
                        case('FROM'):
                            foreach ($msg[$match_type] as $address) {
                                $address[0] = decodeHeader($address[0], true, false);
                                $address[1] = decodeHeader($address[1], true, false);
                                if (strstr('^^' . strtolower($address[0]), $high_val) ||
                                    strstr('^^' . strtolower($address[1]), $high_val)) {
                                    $hlt_color = $message_highlight_list_part['color'];
                                    break 4;
                                }
                            }
                            break;
                        default:
                            $headertest = strtolower(decodeHeader($msg[$match_type], true, false));
                            if (strstr('^^' . $headertest, $high_val)) {
                                $hlt_color = $message_highlight_list_part['color'];
                                break 3; 
                            }
                            break;
                    }
                }
            }
        }
    }

    if (!isset($hlt_color)) {
        $hlt_color = $color_string;
    }

    $col = 0;

    $subject = decodeHeader($msg['SUBJECT']);
    $subject = decodeHTML($subject);
    $subject = processSubject($subject, $indent_array[$msg['ID']],$max_char_obj_m);
    $subject = codeHTML($subject);

    $senderName = processSender($senderName);
    $senderName = codeHTML($senderName);

    echo '<tr '.$style.' class="handcursor" id="tr'.$t.'" bgcolor="'.$hlt_color.'" onClick="OpenPrintMessage('.$msg["ID"].','.$t.',\''.$msg['folder_complete'].'\');" valign="top">'."\n";

    if (sizeof($index_order)) {
        foreach ($index_order as $index_order_part) {
            switch ($index_order_part) {
            case 1: /* checkbox */
		echo '<td align="center" width="20">'.addCheckBox('msg['.$t.']', FALSE, $msg['ID']).'</td>';
                break;
            case 2: /* from */
		$td_str=$senderName;
		if (!$td_str) { $td_str = "&nbsp;"; }
		echo '<td width="25%" nowrap=1>'.$td_str.'</td>';
                break;
            case 3: /* date */
                $date_string = $msg['DATE_STRING'] . '';
                if ($date_string == '') {
                    $date_string = _("Unknown date");
                }
		$td_str=codeHTML($date_string);
		if (!$td_str) { $td_str = "&nbsp;"; }
		echo '<td nowrap=1 width="15%">'.$td_str.'</td>';
                break;
            case 4: /* subject */
		$td_str="";
                if ($thread_sort_messages == 1) {
                    if (isset($indent_array[$msg['ID']])) {
                        $td_str = str_repeat("&nbsp;&nbsp;&nbsp;&nbsp;",$indent_array[$msg['ID']]);
                    }
                }
                $td_str .= "$subject";
		if (!$td_str) { $td_str = "&nbsp;"; }
		echo '<td>'.$td_str.'</td>';
                break;
            case 5: /* flags */
                $stuff = false;
                $td_str = "";
		    $msg_icon = '';
		    $msg_title='';
		    $msg_alt='';	
            	    if (!isset($msg['FLAG_SEEN']) || ($msg['FLAG_SEEN']) == false) {
                	$msg_alt = _('(New)');
                	$msg_title = _('(New)');
                	$msg_icon .= $base_uri.'images/'.$custom_css.'/msg_new';
            	    } else {
                	$msg_alt = _('(Read)');
                	$msg_title = _('(Read)');
                	$msg_icon .= $base_uri.'images/'.$custom_css.'/msg_read';
            	    }
		    $td_str .= '<IMG id="img'.$t.'" SRC="' . $msg_icon . '.gif" border="0" alt="'. $msg_alt . '" title="' .$msg_title . '" >';
                if (!$stuff) {
                    $td_str .= '&nbsp;';
                }
                echo html_tag( 'td',
                               $td_str,
                               'center',
                               '',
                               'nowrap=1 width="20"' );
                break;
            case 6: /* size */
                break;
            case 7: /* priority */
                if ($default_use_priority) {
                    if ( ($msg['PRIORITY'] == 1) || ($msg['PRIORITY'] == 2) ) {
			$td_str = '<IMG SRC="'.$base_uri.'images/'.$custom_css.'/prio_high.gif" border="0"> '; 
                    } else if ( ($msg['PRIORITY'] == 5) ) {
			$td_str = '<IMG SRC="'.$base_uri.'images/'.$custom_css.'/prio_low.gif" border="0"> '; 
                    } else {
			$td_str = '<IMG SRC="'.$base_uri.'images/'.$custom_css.'/transparent.gif" border="0" width="5"> ';
                    }
                }
                echo html_tag( 'td',$td_str,'center','','nowrap=1 width="20"' );
                break;
            case 8: /* attach */
                if ($msg['TYPE0'] == 'multipart') {
		    $td_str = '<IMG SRC="'.$base_uri.'images/'.$custom_css.'/attach.gif" border="0">'; 
                } else {
		    $td_str = '<IMG SRC="'.$base_uri.'images/'.$custom_css.'/transparent.gif" border="0" width="6">';
		}
                echo html_tag( 'td',$td_str,'center','','nowrap=1 width="20"' );
                break;
            }
            ++$col;
        }
	echo '<td width="15%" nowrap=1>'.$msg['folder'].'</td>'."\n";
	echo addHidden('mailboxes['.$t.']', $msg['folder_complete']);
    }
    if ($not_last) {
        echo '</tr>' . "\n" . '<tr><td COLSPAN="' . $col . '" HEIGHT="1"></td></tr>' . "\n";
    } else {
        echo '</tr>'."\n";
    }
}

function printHeaderSearch() {

    global $index_order, $base_uri, $custom_css;

    echo '<tr>'."\n";
    
    foreach ($index_order as $item) {
        switch ($item) {
        case 1: /* checkbox */
            echo '<td onClick="parent.opener.CheckAll(document.FormSearch)" align="center" width="20"><img alt="'._("Toggle All").'" src="'.$base_uri.'images/'.$custom_css.'/checkbox.gif"></td>';
            break;
        case 2: /* from */
            echo html_tag( 'td' ,'' , 'left', '', 'width="25%"' )
                     . '<b>' . codeHTML(_("To/From")) . '</b>';
            break;
        case 3: /* date */
            echo html_tag( 'td' ,'' , 'left', '', 'width="15%" nowrap' )
                 . '<b>' . codeHTML(_("Date")) . '</b>';
            echo "</td>\n";
            break;
        case 4: /* subject */
    	    echo html_tag( 'td' ,'' , 'left', '', '' )
                 . '<b>' . codeHTML(_("Subject")) . '</b>';
            echo "</td>\n";
            break;
        case 5: /* flags */
            echo '<td align="center" width="20"><img src="'.$base_uri.'images/'.$custom_css.'/msg_new.gif"></td>';
            break;
        case 6: /* size */
            break;
        case 7: /* priority */
            echo '<td align="center" width="20"><img src="'.$base_uri.'images/'.$custom_css.'/prio_high.gif"></td>';
            break;
        case 8: /* attach */
            echo '<td align="center" width="20"><img src="'.$base_uri.'images/'.$custom_css.'/attach.gif"></td>';
            break;
        }
    }
    echo '<td width="15%" nowrap align="center"><b>' . codeHTML(_("Folder")) . '</b></td>';
    echo "</tr>\n";
}

function NavigationSearch() {
    global $hide_sm_attributions, $PHP_SELF, $frame_top, $icon_folder,
           $compose_new_win, $auto_expunge, $patermailbox,
           $attachemessages, $provider_name, $provider_uri, $base_uri, 
           $javascript_on, $default_use_mdn, $mdn_user_support, $custom_css;

    sqgetGlobalVar('base_uri', $base_uri, SQ_SESSION );
    sqgetGlobalVar('delimiter', $delimiter, SQ_SESSION );
    echo '<table align="left" valign="top" width="100%" height="100%" cellspacing="0" cellpadding="0">';
		    echo '<tr height="100%">';
		        echo '<td class="nav" nowrap=1 height="100%" align="center" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'" onclick="if (CtrlChecked()) { 
			new_win_overlook(\'move\',\''.$base_uri.'src/move_email.php?search=1\',400,333,1,180,150); }">';
			    if ($icon_folder) {
				echo '<img src="'.$base_uri.'images/'.$custom_css.'/mail_move.gif" border=0>';
				echo '<br>';
			    }
			    echo codeHTML(_("Move"));
			echo '</td>'."\n";		
		        echo '<td class="nav" nowrap=1 height="100%" align="center" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'" onclick="if (CtrlChecked()) { overlook_Action(\'markRead\'); }">';
			    if ($icon_folder) {
				echo '<img src="'.$base_uri.'images/'.$custom_css.'/read.gif" border=0>';
				echo '<br>';
			    }
			    echo codeHTML(_("Read"));
			echo '</td>'."\n";		
		        echo '<td class="nav" nowrap=1 height="100%" align="center" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'" onclick="if (CtrlChecked()) { overlook_Action(\'markUnread\'); }">';
			    if ($icon_folder) {
				echo '<img src="'.$base_uri.'images/'.$custom_css.'/unread.gif" border=0>';
				echo '<br>';
			    }
			    echo codeHTML(_("Unread"));
			echo '</td>'."\n";
			if (!isset($passed_ent_id)) { $passed_ent_id=""; }		
			if (!isset($passed_id)) { $passed_id=""; }		
			if (!isset($mailbox)) { $mailbox=""; }		
			$params='passed_ent_id=' . $passed_ent_id .
            		    '&mailbox=' . urlencode($mailbox) .
            		    '&passed_id=' . $passed_id;
			echo '<td align="left" width="1px" class="navseparator"></td>';
		        echo '<td class="nav" nowrap=1 height="100%" align="center" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'" onClick="if (CtrlChecked()) { parent.message_id_selected=0; overlook_Action(\'delete\'); }">';
			    if ($icon_folder) {
				echo '<img src="'.$base_uri.'images/'.$custom_css.'/delete.gif" border=0>';
				echo '<br>';
			    }
			    echo codeHTML(_("Delete"));
			echo '</td>'."\n";		
			if (!$auto_expunge) { 
		    	    echo '<td class="nav" nowrap=1 align="center" onmouseover="this.className=\'navclicked\'" onmouseout="this.className=\'nav\'" onClick="if (CtrlChecked()) { overlook_Action(\'expungeButton\'); }">';
				if ($icon_folder) {
				    echo '<img src="'.$base_uri.'images/'.$custom_css.'/mail_move.gif" border=0>';
				    echo '<br>';
				}
				echo codeHTML(_("Expunge")) .'&nbsp;' . codeHTML(_("mailbox"));
			    echo '</td>'."\n";		
			}
		    echo "</tr>\n";
    echo  "</table>\n";
}

/* ------------------------ main ------------------------ */

/*  reset these arrays on each page load just in case  */
$attributes = array ();
$saved_attributes = array ();
$search_all = 'none';
$perbox_count = array ();
$recent_count = getPref($data_dir, $username, 'search_memory', 0);

/*  get mailbox names  */
$imapConnection = sqimap_login($username, $key, $imapServerAddress, $imapPort, 0);
$boxes = sqimap_mailbox_list($imapConnection);

/*  set current mailbox to INBOX if none was selected or if page
    was called to search all folders.  */
if ( !isset($mailbox) || $mailbox == 'None' || $mailbox == '' ) {
    $mailbox = $boxes[0]['unformatted'];
}

/*  See how the page was called and fire off correct function  */
if ((!isset($submit) || empty($submit)) && !empty($what)) {
    $submit = _("Search");
}
if ( !isset( $submit ) ) {
    $submit = '';
} else if ($submit == _("Search") && !empty($what)) {
    if ($recent_count > 0) {
        update_recent($what, $where, $mailbox, $username, $data_dir);
    }
} else if ($submit == 'forget') {
    forget_recent($count, $username, $data_dir);
} else if ($submit == 'save') {
    save_recent($count, $username, $data_dir);
} else if ($submit == 'delete') {
    delete_saved($count, $username, $data_dir);
}

//********************** START HTML *******************************//

$js = "\n".'<script language="javascript">
    var custom_css = "'.$custom_css.'";
    var delimiter = "'.$delimiter.'";
    var base_uri = "'.$base_uri.'";
    var select_message_onclick = "'.$select_message_onclick.'";
    var patermailbox = "'.$patermailbox.'";
    var mailbox = "'.$mailbox.'";
    var start_msg = "'.$start_msg.'";
    var msgread = "'.jsspecialchars(_('(Read)')).'";
    var msgselectfolder = "'.jsspecialchars(_("Please select a folder!")).'";
    var msginsertsearch = "'.jsspecialchars(_("Please insert a string of search")).'";
    var msgnomsg = "'.jsspecialchars(_('No messages were selected.')).'";
</script>'."\n";

displayHtmlHeader(_("Search"),$js);

set_colors_js_IEbug("BODY",".selectedmsg");

echo '<script language="javascript" type="text/javascript" src="'.$base_uri.'js/search.js"></script>'."\n";
echo '<script language="javascript" type="text/javascript" src="'.$base_uri.'js/new_window.js"></script>'."\n";

echo '<body onload="self.focus();">';

echo '<table class="border" height="100%" valign="top" width="100%" cellpadding="0" cellspacing="0" border="0">';

//do_hook('search_before_form');

/*  update the recent and saved searches from the pref files  */
$attributes = get_recent($username, $data_dir);
$saved_attributes = get_saved($username, $data_dir);
$saved_count = count($saved_attributes['saved_what']);
$count_all = 0;

/* Saved Search Table */
if ($saved_count > 0) {
    echo '<tr valign="middle"><td width="100%" height="1%" valign="middle">';  
    echo '<table class="border" width="100%" align="center" valign="middle" cellpadding="0" cellspacing="2" border="0">	
	    <tr valign="middle">
		<td width="100%" colspan=4 height="1%" align="left" class="optionlink">
		    '.codeHTML(_("Saved Searches")).'
		</td>
	    </tr>';	
    for ($i=0; $i < $saved_count; ++$i) {
        if ($i % 2) {
	    echo '<tr valign="middle" bgcolor="'.$color[0].'">';
        } else {
	    echo '<tr valign="middle" bgcolor="'.$color[4].'">';
        }
	echo '<td width="35%" height="1%" align="left" class="nav">
		'.codeHTML(imap_utf7_decode_local($saved_attributes['saved_folder'][$i])).'
	      </td>	
	      <td height="1%" align="left" class="nav">
		'.codeHTML($saved_attributes['saved_what'][$i]).'
	      </td>
	      <td height="1%" align="center" class="nav">
		'.codeHTML($saved_attributes['saved_where'][$i]).'
	      </td>
	      <td height="1%" align="right" class="nav">
    		<a href='.$base_uri.'src/search.php?mailbox=' . htmlspecialchars($saved_attributes['saved_folder'][$i])
    		    .'&what=' . htmlspecialchars($saved_attributes['saved_what'][$i])
    		    .'&where=' . htmlspecialchars($saved_attributes['saved_where'][$i]).'>'. codeHTML(_("edit"))
		    .'</a>&nbsp;|&nbsp;
    		<a href='.$base_uri.'src/search.php?mailbox=' . urlencode($saved_attributes['saved_folder'][$i])
    		    .'&what=' . urlencode($saved_attributes['saved_what'][$i])
    		    .'&where=' . urlencode($saved_attributes['saved_where'][$i])
    		    .'&submit=Search_no_update>' . codeHTML(_("search")).'
		</a>&nbsp;|&nbsp
    		<a href='.$base_uri.'src/search.php?count='.$i.'&submit=delete>'.codeHTML(_("delete")).'</a>
	      </td>
	     </tr>';
    }	      
    echo '</table></td></tr>'."\n";
}

if ($recent_count > 0) {
    echo '<tr valign="middle"><td width="100%" height="1%" valign="middle">';  
    echo '<table class="border" width="100%" align="center" valign="middle" cellpadding="0" cellspacing="2" border="0">	
	    <tr valign="middle">
		<td width="100%" height="1%" align="left" class="optionlink">
		    '.codeHTML(_("Recent Searches")).'
		</td>
	    </tr>';	
    for ($i=1; $i <= $recent_count; ++$i) {
        if (isset($attributes['search_folder'][$i])) { 
    	    if ($attributes['search_folder'][$i] == "") {
        	$attributes['search_folder'][$i] = "INBOX";
    	    }
        }
        if ($i % 2) {
	    echo '<tr valign="middle" bgcolor="'.$color[0].'">';
        } else {
	    echo '<tr valign="middle" bgcolor="'.$color[4].'">';
        }
    if (isset($attributes['search_what'][$i]) && !empty($attributes['search_what'][$i])) {
	echo '<td width="35%" height="1%" align="left" class="nav">
		'.codeHTML(imap_utf7_decode_local($attributes['search_folder'][$i])).'
	      </td>	
	      <td height="1%" align="left" class="nav">
		'.codeHTML($attributes['search_what'][$i]).'
	      </td>
	      <td height="1%" align="center" class="nav">
		'.codeHTML($attributes['search_where'][$i]).'
	      </td>
	      <td height="1%" align="right" class="nav">
		    <a href='.$base_uri.'src/search.php?count='.$i.'&submit=save>'.codeHTML(_("save")).'</a>
		    &nbsp;|&nbsp; 
            	    <a href='.$base_uri.'src/search.php?mailbox=' . urlencode($attributes['search_folder'][$i])
            		.'&what=' . urlencode($attributes['search_what'][$i])
            		.'&where=' . urlencode($attributes['search_where'][$i])
            		.'&submit=Search_no_update>' . codeHTML(_("search")) . '
		    </a>&nbsp;|&nbsp;
            	    <a href='.$base_uri.'src/search.php?count='.$i.'&submit=forget>'.codeHTML(_("forget")).'</a>
	        </td>
	     </tr>';
    }
    }
    echo '</table></td></tr>'."\n";
}


if (isset($newsort)) {
    $sort = $newsort;
    sqsession_register($sort, 'sort');
}

/*********************************************************************
 * Check to see if we can use cache or not. Currently the only time  *
 * when you will not use it is when a link on the left hand frame is *
 * used. Also check to make sure we actually have the array in the   *
 * registered session data.  :)                                      *
 *********************************************************************/
if (! isset($use_mailbox_cache)) {
    $use_mailbox_cache = 0;
}

/* There is a problem with registered vars in 4.1 */
/*
if( substr( phpversion(), 0, 3 ) == '4.1'  ) {
    $use_mailbox_cache = FALSE;
}
*/

/************************* Search Form *******************************/
    echo addForm($base_uri.'src/search.php', 'POST', 'sform');
    echo '<tr valign="top"><td width="100%" height="1%" valign="top">';  
    if (isset($subdir) && $subdir) { $checked=TRUE; } else { $checked=FALSE; }
    echo '<table class="border" width="100%" align="center" valign="top" cellpadding="0" cellspacing="2" border="0">	
	    <tr valign="middle">
		<td width="100%" height="1%" align="left" class="optionlink">
		    '.codeHTML(_("Current Search")).'
		</td>
	    </tr>
	    <tr valign="middle">
		<td width="100%" height="1%" align="left" class="nav">
		    '.codeHTML(_("Search into")).'&nbsp;:';
		    echo addHidden('mailbox', $patermailbox).'&nbsp;';	
		    echo addInput('mailbox_label', $showpatermailbox, '45', '','disabled=1').'&nbsp;';
		    echo addCheckBox('subdir', $checked, 1).'&nbsp;';
		    echo codeHTML(_("include subfolders")).'&nbsp;';
		    echo addButton(_("Other folders"),'', '','new_win_overlook("move_email","'.$base_uri.'src/search_folder.php",400,333,1,180,150);','sform','',1,1);
		echo '</td>
	    </tr>'."\n";	
    
if ( !isset( $what ) ) {
    $what = '';
}
if ( !isset( $where ) ) {
    $where = 'FROM';
}

$what_disp = str_replace(',', ' ', $what);
$what_disp = str_replace('\\\\', '\\', $what_disp);
$what_disp = str_replace('\\"', '"', $what_disp);
$what_disp = str_replace('"', '&quot;', $what_disp);

	echo '<tr valign="top">
		<td width="100%" height="1%" align="left" class="nav">';
	    $new_option = array('BODY' => _("Body"),
				'TEXT' => _("Everywhere"),
				'SUBJECT' => _("Subject"),
				'FROM' => _("From"),
				'CC' => _("Cc"),
				'TO' => _("To"));
	    echo addSelect('where', $new_option, $where, TRUE, 'style="width:200px" size=1');
	echo '	    &nbsp;';
	    echo addInput('what', $what_disp, '60');
	echo '	</td>
	    </tr>'."\n";	
	echo '<tr valign="top">
		<td width="100%" height="1%" align="center" class="nav">';
	    echo addHidden('submitsearch',_("Search"));
	    echo addButton(_("Close"), '', '', 'window.close();','sform','',1,1);
	    echo '&nbsp;&nbsp;&nbsp;';
	    echo addButton(_("Search"),'','','','sform','OnSubmitIT()');
	echo '</td>
	    </tr>'."\n";	
    echo '</table></td></tr></form>'."\n";

//do_hook('search_after_form');

flush();

/***************************************************************************
    search all folders option still in the works. returns a table for each
    folder it finds a match in.
*****************************************************************************/

$old_value = 0;
if ($allow_thread_sort == TRUE) {
    $old_value = $allow_thread_sort;
    $allow_thread_sort = FALSE;
}

// ricerca anche sulle sottocartelle
if (isset($subdir) && $subdir) {

    $boxcount = count($boxes);

    echo '<tr valign="top"><td width="100%" height="1%" valign="top">';  
    echo '<table class="border" width="100%" align="center" valign="top" cellpadding="0" cellspacing="2" border="0">	
	    <tr valign="middle">
		<td width="100%" height="1%" align="left" class="optionlink">
		    '.codeHTML(_("Search Results")).'
		</td>
	    </tr>';
    $msgs=array();
    $count_all=0;
    for ($x=0;$x<$boxcount;$x++) {
        if (!in_array('noselect', $boxes[$x]['flags'])) {
            $mailbox = $boxes[$x]['unformatted'];
        }
        if ($mailbox && preg_match("/^$patermailbox$delimiter/", $mailbox) || $mailbox == $patermailbox) {
            sqimap_mailbox_select($imapConnection, $mailbox);
            $msgs_partial = sqimap_search($imapConnection, $where, $what, $mailbox, $color, 0, $search_all, $count_all);
	    $count_all += count($msgs_partial);
	    if ( $mailbox == 'INBOX' ) {
    		$showbox = _("INBOX");
	    } else {
    		$showbox = imap_utf7_decode_local($mailbox);
    		$a = explode($delimiter,$showbox);
    		$showbox = array_pop($a);
	    }
	    for ($a=0;$a<count($msgs_partial);$a++) {
		$msgs_partial[$a]["folder"]=$showbox;
		$msgs_partial[$a]["folder_complete"]=$mailbox;
		array_push($msgs,$msgs_partial[$a]);
	    }
        }
	$mailbox="";
    }
    if ($count_all) {
	echo '<tr valign="middle">
	 	<td width="100%" height="1%" align="left" class="optionlink">';
		NavigationSearch();
	echo '</td></tr>'."\n";
	printSearchMessages($msgs, $count_all, $imapConnection, 
	                    $where, $what, false, false);
    } else if (!$count_all) {
	    echo '<tr valign="middle">
		    <td width="100%" height="1%" align="left" class="nav">
			'.codeHTML(_("No Messages Found")).'
		    </td>
		</tr>'."\n";	
    }
    echo '</table></td></tr>'."\n";  
/*  ricerca su una singola cartella  */
} else {
    if (($submit == _("Search") || $submit == 'Search_no_update') && !empty($what)) {
        sqimap_mailbox_select($imapConnection, $mailbox);
        $msgs = sqimap_search($imapConnection, $where, $what, $mailbox, $color, 0, $search_all, $count_all);
	if ( $mailbox == 'INBOX' ) {
    	    $showbox = _("INBOX");
	} else {
    	    $showbox = imap_utf7_decode_local($mailbox);
    	    $a = explode($delimiter,$showbox);
    	    $showbox = array_pop($a);
	}
	for ($a=0;$a<count($msgs);$a++) {
	    $msgs[$a]["folder"]=$showbox;
	    $msgs[$a]["folder_complete"]=$mailbox;
	}

	echo '<tr valign="top"><td width="100%" height="1%" valign="top">';  
	echo '<table class="border" width="100%" align="center" valign="top" cellpadding="0" cellspacing="2" border="0">	
		<tr valign="middle">
		    <td width="100%" height="1%" align="left" class="optionlink">
			'.codeHTML(_("Search Results")).'
		    </td>
		</tr>';
	if (count($msgs)) {
	    echo '<tr valign="middle">
		    <td width="100%" height="1%" align="left" class="optionlink">';
			NavigationSearch();
	    echo '</td></tr>'."\n";
            printSearchMessages($msgs, count($msgs), $imapConnection,
	                       $where, $what, false, false);
	} else {
	    echo '<tr valign="middle">
		    <td width="100%" height="1%" align="left" class="nav">
			'.codeHTML(_("No Messages Found")).'
		    </td>
		</tr>'."\n";	
	}
	echo '</table></td></tr>'."\n";
    }
}

echo '<tr><td class="nav" width="100%" align="left">&nbsp;</td></tr>'."\n";

$allow_thread_sort = $old_value;

//do_hook('search_bottom');

sqimap_logout($imapConnection);
echo '</table></body></html>';

?>
